@@ -35,7 +35,7 @@ then
   then
     cp -v storage/columnstore/columnstore/debian/mariadb-plugin-columnstore.* debian/
     echo >> debian/control
-    sed "s/10.6/${MYSQL_VERSION_MAJOR}.${MYSQL_VERSION_MINOR}/" <storage/columnstore/columnstore/debian/control >> debian/control
+    sed "s/-10.6//" <storage/columnstore/columnstore/debian/control >> debian/control
   fi
 fi
 
@@ -76,15 +76,8 @@ disable_libfmt()
 
 architecture=$(dpkg-architecture -q DEB_BUILD_ARCH)
 
-LSBID="$(lsb_release -si  | tr '[:upper:]' '[:lower:]')"
-LSBVERSION="$(lsb_release -sr | sed -e "s#\.##g")"
-LSBNAME="$(lsb_release -sc)"
-
-if [ -z "${LSBID}" ]
-then
-    LSBID="unknown"
-fi
-case "${LSBNAME}" in
+CODENAME="$(lsb_release -sc)"
+case "${CODENAME}" in
   stretch)
     # MDEV-16525 libzstd-dev-1.1.3 minimum version
     sed -e '/libzstd-dev/d' \
@@ -143,7 +136,7 @@ case "${LSBNAME}" in
     fi
     ;;
   *)
-    echo "Error - unknown release codename $LSBNAME" >&2
+    echo "Error - unknown release codename $CODENAME" >&2
     exit 1
 esac
 
@@ -160,9 +153,9 @@ UPSTREAM="${MYSQL_VERSION_MAJOR}.${MYSQL_VERSION_MINOR}.${MYSQL_VERSION_PATCH}${
 PATCHLEVEL="+maria"
 LOGSTRING="MariaDB build"
 EPOCH="1:"
-VERSION="${EPOCH}${UPSTREAM}${PATCHLEVEL}~${LSBID:0:3}${LSBVERSION}"
+VERSION="${EPOCH}${UPSTREAM}${PATCHLEVEL}~${CODENAME}"
 
-dch -b -D ${LSBNAME} -v "${VERSION}" "Automatic build with ${LOGSTRING}." --controlmaint
+dch -b -D "${CODENAME}" -v "${VERSION}" "Automatic build with ${LOGSTRING}." --controlmaint
 
 echo "Creating package version ${VERSION} ... "
 
