@@ -29,12 +29,12 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1335  USA
 
 struct xb_delta_info_t
 {
-	xb_delta_info_t(ulint page_size, ulint zip_size, ulint space_id)
+	xb_delta_info_t(ulint page_size, ulint zip_size, uint32_t space_id)
 	: page_size(page_size), zip_size(zip_size), space_id(space_id) {}
 
 	ulint	page_size;
 	ulint	zip_size;
-	ulint	space_id;
+	uint32_t space_id;
 };
 
 class CorruptedPages
@@ -42,23 +42,23 @@ class CorruptedPages
 public:
   CorruptedPages();
   ~CorruptedPages();
-  void add_page(const char *file_name, ulint space_id, unsigned page_no);
-  bool contains(ulint space_id, unsigned page_no) const;
-  void drop_space(ulint space_id);
-  void rename_space(ulint space_id, const std::string &new_name);
+  void add_page(const char *file_name, page_id_t page_id);
+  bool contains(page_id_t page_id) const;
+  void drop_space(uint32_t space_id);
+  void rename_space(uint32_t space_id, const std::string &new_name);
   bool print_to_file(const char *file_name) const;
   void read_from_file(const char *file_name);
   bool empty() const;
   void zero_out_free_pages();
 
 private:
-  void add_page_no_lock(const char *space_name, ulint space_id,
-                        unsigned page_no, bool convert_space_name);
+  void add_page_no_lock(const char *space_name, page_id_t page_id,
+                        bool convert_space_name);
   struct space_info_t {
     std::string space_name;
-    std::set<unsigned> pages;
+    std::set<uint32_t> pages;
   };
-  typedef std::map<ulint, space_info_t> container_t;
+  typedef std::map<uint32_t, space_info_t> container_t;
   mutable pthread_mutex_t m_mutex;
   container_t m_spaces;
 };
@@ -71,6 +71,7 @@ extern char		*xtrabackup_incremental_dir;
 extern char		*xtrabackup_incremental_basedir;
 extern char		*innobase_data_home_dir;
 extern char		*innobase_buffer_pool_filename;
+extern char		*buffer_pool_filename;
 extern char		*xb_plugin_dir;
 extern char		*xb_rocksdb_datadir;
 extern my_bool	xb_backup_rocksdb;
@@ -79,9 +80,6 @@ extern uint		opt_protocol;
 extern ds_ctxt_t	*ds_meta;
 extern ds_ctxt_t	*ds_data;
 
-/* The last checkpoint LSN at the backup startup time */
-extern lsn_t checkpoint_lsn_start;
-
 extern xb_page_bitmap *changed_page_bitmap;
 
 extern char		*xtrabackup_incremental;
@@ -166,6 +164,7 @@ extern uint		opt_lock_wait_timeout;
 extern uint		opt_lock_wait_threshold;
 extern uint		opt_debug_sleep_before_unlock;
 extern uint		opt_safe_slave_backup_timeout;
+extern uint		opt_max_binlogs;
 
 extern const char	*opt_history;
 
@@ -235,4 +234,65 @@ typedef void (*insert_entry_func_t)(const char*);
 void xb_load_list_string(char *list, const char *delimiters,
                          insert_entry_func_t ins);
 void register_ignore_db_dirs_filter(const char *name);
+
+#ifdef _WIN32
+typedef HANDLE	os_file_dir_t;	/*!< directory stream */
+/** The os_file_opendir() function opens a directory stream corresponding to the
+directory named by the dirname argument. The directory stream is positioned
+at the first entry. In both Unix and Windows we automatically skip the '.'
+and '..' items at the start of the directory listing.
+
+@param[in]	dirname		directory name; it must not contain a trailing
+				'\' or '/'
+@return directory stream
+@retval INVALID_HANDLE_VALUE on error */
+HANDLE os_file_opendir(const char *dirname);
+# define os_file_closedir(dir) static_cast<void>(FindClose(dir))
+# define os_file_closedir_failed(dir) !FindClose(dir)
+#else
+typedef DIR* os_file_dir_t;
+# define os_file_opendir(dirname) opendir(dirname)
+# define os_file_closedir(dir) static_cast<void>(closedir(dir))
+# define os_file_closedir_failed(dir) closedir(dir)
+#endif
+
+/** This function returns information of the next file in the directory. We jump
+over the '.' and '..' entries in the directory.
+@param[in]	dirname		directory name or path
+@param[in]	dir		directory stream
+@param[out]	info		buffer where the info is returned
+@return 0 if ok, -1 if error, 1 if at the end of the directory */
+int
+os_file_readdir_next_file(
+	const char*	dirname,
+	os_file_dir_t	dir,
+	os_file_stat_t* info);
+
+/***********************************************************************//**
+A fault-tolerant function that tries to read the next file name in the
+directory. We retry 100 times if os_file_readdir_next_file() returns -1. The
+idea is to read as much good data as we can and jump over bad data.
+@return 0 if ok, -1 if error even after the retries, 1 if at the end
+of the directory */
+int
+fil_file_readdir_next_file(
+/*=======================*/
+	dberr_t*	err,	/*!< out: this is set to DB_ERROR if an error
+				was encountered, otherwise not changed */
+	const char*	dirname,/*!< in: directory name or path */
+	os_file_dir_t	dir,	/*!< in: directory stream */
+	os_file_stat_t* info);	/*!< in/out: buffer where the
+				info is returned */
+
+#ifndef DBUG_OFF
+#include <fil0fil.h>
+extern void dbug_mariabackup_event(const char *event,
+                            const fil_space_t::name_type key);
+
+#define DBUG_MARIABACKUP_EVENT(A, B)                                          \
+  DBUG_EXECUTE_IF("mariabackup_events", dbug_mariabackup_event(A, B);)
+#else
+#define DBUG_MARIABACKUP_EVENT(A, B) /* empty */
+#endif // DBUG_OFF
+
 #endif /* XB_XTRABACKUP_H */
