@@ -19,32 +19,14 @@
 #include "sql_priv.h"
 #include "sql_class.h" /* TMP_TABLE_PARAM */
 #include "table.h"
-#include "sql_type_json.h"
 #include "item_jsonfunc.h"
 #include "json_table.h"
 #include "sql_show.h"
 #include "sql_select.h"
 #include "create_tmp_table.h"
-#include "sql_parse.h"
 
 #define HA_ERR_JSON_TABLE (HA_ERR_LAST+1)
 
-/*
-  Allocating memory and *also* using it (reading and
-  writing from it) because some build instructions cause
-  compiler to optimize out stack_used_up. Since alloca()
-  here depends on stack_used_up, it doesnt get executed
-  correctly and causes json_debug_nonembedded to fail
-  ( --error ER_STACK_OVERRUN_NEED_MORE does not occur).
-*/
-#define ALLOCATE_MEM_ON_STACK(A) do \
-                              { \
-                                uchar *array= (uchar*)alloca(A); \
-                                array[0]= 1; \
-                                array[0]++; \
-                                array[0] ? array[0]++ : array[0]--; \
-                              } while(0)
-
 class table_function_handlerton
 {
 public:
@@ -118,15 +100,6 @@ int get_disallowed_table_deps_for_list(MEM_ROOT *mem_root,
   NESTED_JOIN *nested_join;
   List_iterator<TABLE_LIST> li(*join_list);
 
-  DBUG_EXECUTE_IF("json_check_min_stack_requirement",
-                  {
-                    long arbitrary_var;
-                    long stack_used_up= (available_stack_size(current_thd->thread_stack, &arbitrary_var));
-                    ALLOCATE_MEM_ON_STACK(my_thread_stack_size-stack_used_up-STACK_MIN_SIZE);
-                  });
-  if (check_stack_overrun(current_thd, STACK_MIN_SIZE , NULL))
-    return 1;
-
   while ((table= li++))
   {
     if ((nested_join= table->nested_join))
@@ -339,7 +312,8 @@ handle_new_nested:
 
   while (!json_get_path_next(&m_engine, &m_cur_path))
   {
-    if (json_path_compare(&m_path, &m_cur_path, m_engine.value_type))
+    if (json_path_compare(&m_path, &m_cur_path, m_engine.value_type,
+                          NULL))
       continue;
     /* path found. */
     ++m_ordinality_counter;
@@ -404,25 +378,6 @@ static void store_json_in_field(Field *f, const json_engine_t *je)
 }
 
 
-static int store_json_in_json(Field *f, json_engine_t *je)
-{
-  const uchar *from= je->value_begin;
-  const uchar *to;
-
-  if (json_value_scalar(je))
-    to= je->value_end;
-  else
-  {
-    int error;
-    if ((error= json_skip_level(je)))
-      return error;
-    to= je->s.c_str;
-  }
-  f->store((const char *) from, (uint32) (to - from), je->s.cs);
-  return 0;
-}
-
-
 bool Json_table_nested_path::check_error(const char *str)
 {
   if (m_engine.s.error)
@@ -547,7 +502,7 @@ int ha_json_table::fill_column_values(THD *thd, uchar * buf, uchar *pos)
       {
         json_engine_t je;
         json_path_step_t *cur_step;
-        uint array_counters[JSON_DEPTH_LIMIT];
+        int array_counters[JSON_DEPTH_LIMIT];
         int not_found;
         const uchar* node_start;
         const uchar* node_end;
@@ -587,12 +542,7 @@ int ha_json_table::fill_column_values(THD *thd, uchar * buf, uchar *pos)
           }
           else
           {
-            if (jc->m_format_json)
-            {
-              if (!(error= store_json_in_json(*f, &je)))
-                error= er_handler.errors;
-            }
-            else if (!(error= !json_value_scalar(&je)))
+            if (!(error= !json_value_scalar(&je)))
             {
               store_json_in_field(*f, &je);
               error= er_handler.errors;
@@ -611,7 +561,8 @@ int ha_json_table::fill_column_values(THD *thd, uchar * buf, uchar *pos)
                 more matches for it in json and report an error if so.
               */
               if (jc->m_path.types_used &
-                    (JSON_PATH_WILD | JSON_PATH_DOUBLE_WILD) &&
+                    (JSON_PATH_WILD | JSON_PATH_DOUBLE_WILD |
+                     JSON_PATH_ARRAY_RANGE) &&
                   (json_scan_next(&je) ||
                    !json_find_path(&je, &jc->m_path, &cur_step,
                                    array_counters)))
@@ -767,7 +718,7 @@ bool Create_json_table::add_json_table_fields(THD *thd, TABLE *table,
   uint fieldnr= 0;
   MEM_ROOT *mem_root_save= thd->mem_root;
   List_iterator_fast<Json_table_column> jc_i(jt->m_columns);
-  Column_derived_attributes da(NULL);
+  Column_derived_attributes da(&my_charset_utf8mb4_general_ci);
   DBUG_ENTER("add_json_table_fields");
 
   thd->mem_root= &table->mem_root;
@@ -784,8 +735,6 @@ bool Create_json_table::add_json_table_fields(THD *thd, TABLE *table,
        executing a prepared statement for the second time.
     */
     sql_f->length= sql_f->char_length;
-    if (!sql_f->charset)
-      sql_f->charset= &my_charset_utf8mb4_general_ci;
 
     if (sql_f->prepare_stage1(thd, thd->mem_root, table->file,
                               table->file->ha_table_flags(), &da))
@@ -920,14 +869,23 @@ int Json_table_column::set(THD *thd, enum_type ctype, const LEX_CSTRING &path,
     anctual content. Not sure though if we should.
   */
   m_path.s.c_str= (const uchar *) path.str;
-
-  if (ctype == PATH)
-    m_format_json= m_field->type_handler() == &type_handler_long_blob_json;
-
   return 0;
 }
 
 
+int Json_table_column::set(THD *thd, enum_type ctype, const LEX_CSTRING &path,
+                           const Lex_charset_collation_st &cl)
+{
+  if (cl.is_empty() || cl.is_contextually_typed_collate_default())
+    return set(thd, ctype, path, nullptr);
+
+  CHARSET_INFO *tmp;
+  if (!(tmp= cl.resolved_to_character_set(&my_charset_utf8mb4_general_ci)))
+    return 1;
+  return set(thd, ctype, path, tmp);
+}
+
+
 static int print_path(String *str, const json_path_t *p)
 {
   return str->append('\'') ||
@@ -970,7 +928,10 @@ int Json_table_column::print(THD *thd, Field **f, String *str)
     if (str->append(column_type) ||
         ((*f)->has_charset() && m_explicit_cs &&
          (str->append(STRING_WITH_LEN(" CHARSET ")) ||
-          str->append(&m_explicit_cs->cs_name))) ||
+          str->append(&m_explicit_cs->cs_name) ||
+          (!(m_explicit_cs->state & MY_CS_PRIMARY) &&
+           (str->append(STRING_WITH_LEN(" COLLATE ")) ||
+            str->append(&m_explicit_cs->coll_name))))) ||
         str->append(m_column_type == PATH ? &path : &exists_path) ||
         print_path(str, &m_path))
       return 1;
@@ -1170,7 +1131,7 @@ bool Table_function_json_table::setup(THD *thd, TABLE_LIST *sql_table,
   // fields in non_agg_field_used:
   const bool saved_non_agg_field_used= s_lex->non_agg_field_used();
 
-  bool res= m_json->fix_fields_if_needed_for_scalar(thd, &m_json);
+  bool res= m_json->fix_fields_if_needed(thd, &m_json);
 
   s_lex->is_item_list_lookup= save_is_item_list_lookup;
   s_lex->set_non_agg_field_used(saved_non_agg_field_used);
@@ -1329,16 +1290,6 @@ static void add_extra_deps(List<TABLE_LIST> *join_list, table_map deps)
 {
   TABLE_LIST *table;
   List_iterator<TABLE_LIST> li(*join_list);
-
-  DBUG_EXECUTE_IF("json_check_min_stack_requirement",
-                  {
-                    long arbitrary_var;
-                    long stack_used_up= (available_stack_size(current_thd->thread_stack, &arbitrary_var));
-                    ALLOCATE_MEM_ON_STACK(my_thread_stack_size-stack_used_up-STACK_MIN_SIZE);
-                  });
-  if (check_stack_overrun(current_thd, STACK_MIN_SIZE , NULL))
-    return;
-
   while ((table= li++))
   {
     table->dep_tables |= deps;
@@ -1427,15 +1378,6 @@ table_map add_table_function_dependencies(List<TABLE_LIST> *join_list,
   table_map res= 0;
   List_iterator<TABLE_LIST> li(*join_list);
 
-  DBUG_EXECUTE_IF("json_check_min_stack_requirement",
-                  {
-                    long arbitrary_var;
-                    long stack_used_up= (available_stack_size(current_thd->thread_stack, &arbitrary_var));
-                    ALLOCATE_MEM_ON_STACK(my_thread_stack_size-stack_used_up-STACK_MIN_SIZE);
-                  });
-  if ((res=check_stack_overrun(current_thd, STACK_MIN_SIZE , NULL)))
-    return res;
-
   // Recursively compute extra dependencies
   while ((table= li++))
   {
