@@ -43,7 +43,8 @@ Master_info::Master_info(LEX_CSTRING *connection_name_arg,
    gtid_reconnect_event_skip_count(0), gtid_event_seen(false),
    in_start_all_slaves(0), in_stop_all_slaves(0), in_flush_all_relay_logs(0),
    users(0), killed(0),
-   total_ddl_groups(0), total_non_trans_groups(0), total_trans_groups(0)
+   total_ddl_groups(0), total_non_trans_groups(0), total_trans_groups(0),
+   is_shutdown(false)
 {
   char *tmp;
   host[0] = 0; user[0] = 0; password[0] = 0;
@@ -85,6 +86,14 @@ Master_info::Master_info(LEX_CSTRING *connection_name_arg,
   mysql_mutex_init(key_master_info_data_lock, &data_lock, MY_MUTEX_INIT_FAST);
   mysql_mutex_init(key_master_info_start_stop_lock, &start_stop_lock,
                    MY_MUTEX_INIT_SLOW);
+  /*
+    start_alter_lock will protect individual start_alter_info while
+    start_alter_list_lock is for list insertion and deletion operations
+  */
+  mysql_mutex_init(key_master_info_start_alter_lock, &start_alter_lock,
+                                      MY_MUTEX_INIT_FAST);
+  mysql_mutex_init(key_master_info_start_alter_list_lock, &start_alter_list_lock,
+                                      MY_MUTEX_INIT_FAST);
   mysql_mutex_setflags(&run_lock, MYF_NO_DEADLOCK_DETECTION);
   mysql_mutex_setflags(&data_lock, MYF_NO_DEADLOCK_DETECTION);
   mysql_mutex_init(key_master_info_sleep_lock, &sleep_lock, MY_MUTEX_INIT_FAST);
@@ -92,6 +101,7 @@ Master_info::Master_info(LEX_CSTRING *connection_name_arg,
   mysql_cond_init(key_master_info_start_cond, &start_cond, NULL);
   mysql_cond_init(key_master_info_stop_cond, &stop_cond, NULL);
   mysql_cond_init(key_master_info_sleep_cond, &sleep_cond, NULL);
+  init_sql_alloc(PSI_INSTRUMENT_ME, &mem_root, MEM_ROOT_BLOCK_SIZE, 0, MYF(0));
 }
 
 
@@ -121,10 +131,13 @@ Master_info::~Master_info()
   mysql_mutex_destroy(&data_lock);
   mysql_mutex_destroy(&sleep_lock);
   mysql_mutex_destroy(&start_stop_lock);
+  mysql_mutex_destroy(&start_alter_lock);
+  mysql_mutex_destroy(&start_alter_list_lock);
   mysql_cond_destroy(&data_cond);
   mysql_cond_destroy(&start_cond);
   mysql_cond_destroy(&stop_cond);
   mysql_cond_destroy(&sleep_cond);
+  free_root(&mem_root, MYF(0));
 }
 
 /**
@@ -170,6 +183,8 @@ void Master_info::clear_in_memory_info(bool all)
   {
     port= MYSQL_PORT;
     host[0] = 0; user[0] = 0; password[0] = 0;
+    domain_id_filter.clear_ids();
+    reset_dynamic(&ignore_server_ids);
   }
 }
 
@@ -747,7 +762,7 @@ int flush_master_info(Master_info* mi,
                          (1 + mi->ignore_server_ids.elements), MYF(MY_WME));
     if (!ignore_server_ids_buf)
       DBUG_RETURN(1);                           /* error */
-    ulong cur_len= sprintf(ignore_server_ids_buf, "%u",
+    ulong cur_len= sprintf(ignore_server_ids_buf, "%zu",
                            mi->ignore_server_ids.elements);
     for (ulong i= 0; i < mi->ignore_server_ids.elements; i++)
     {
@@ -1224,17 +1239,15 @@ bool Master_info_index::init_all_master_info()
 
   if (!err_num) // No Error on read Master_info
   {
-    if (global_system_variables.log_warnings > 1)
+    if (global_system_variables.log_warnings > 2)
       sql_print_information("Reading of all Master_info entries succeeded");
     DBUG_RETURN(0);
   }
-  if (succ_num) // Have some Error and some Success
-  {
-    sql_print_warning("Reading of some Master_info entries failed");
-    DBUG_RETURN(1);
-  }
 
-  sql_print_error("Reading of all Master_info entries failed!");
+  if (succ_num) // Have some Error and some Success
+    sql_print_warning("Reading of some Master_info entries failed");
+  else
+    sql_print_error("Reading of all Master_info entries failed!");
   DBUG_RETURN(1);
 
 error:
@@ -1431,7 +1444,7 @@ bool Master_info_index::add_master_info(Master_info *mi, bool write_to_file)
   if (unlikely(abort_loop) ||
       !my_hash_insert(&master_info_hash, (uchar*) mi))
   {
-    if (global_system_variables.log_warnings > 1)
+    if (global_system_variables.log_warnings > 2)
       sql_print_information("Added new Master_info '%.*s' to hash table",
                             (int) mi->connection_name.length,
                             mi->connection_name.str);
@@ -1455,11 +1468,32 @@ bool Master_info_index::add_master_info(Master_info *mi, bool write_to_file)
    atomic
 */
 
-bool Master_info_index::remove_master_info(Master_info *mi)
+bool Master_info_index::remove_master_info(Master_info *mi, bool clear_log_files)
 {
+  char tmp_name[FN_REFLEN];
   DBUG_ENTER("remove_master_info");
   mysql_mutex_assert_owner(&LOCK_active_mi);
 
+  if (clear_log_files)
+  {
+    /* This code is only executed when change_master() failes to create a new master info */
+
+    // Delete any temporary relay log files that could have been created by change_master()
+    mi->rli.relay_log.reset_logs(current_thd, 0, (rpl_gtid*) 0, 0, 0);
+    /* Delete master-'connection'.info */
+    create_logfile_name_with_suffix(tmp_name,
+                                    sizeof(tmp_name),
+                                    master_info_file, 0,
+                                    &mi->cmp_connection_name);
+    my_delete(tmp_name, MYF(0));
+    /* Delete relay-log-'connection'.info */
+    create_logfile_name_with_suffix(tmp_name,
+                                    sizeof(tmp_name),
+                                    relay_log_info_file, 0,
+                                    &mi->cmp_connection_name);
+    my_delete(tmp_name, MYF(0));
+  }
+
   // Delete Master_info and rewrite others to file
   if (!my_hash_delete(&master_info_hash, (uchar*) mi))
   {
@@ -1797,6 +1831,12 @@ void Domain_id_filter::reset_filter()
   m_filter= false;
 }
 
+void Domain_id_filter::clear_ids()
+{
+  reset_dynamic(&m_domain_ids[DO_DOMAIN_IDS]);
+  reset_dynamic(&m_domain_ids[IGNORE_DOMAIN_IDS]);
+}
+
 /**
   Update the do/ignore domain id filter lists.
 
@@ -1910,7 +1950,7 @@ char *Domain_id_filter::as_string(enum_list_type type)
     return NULL;
 
   // Store the total number of elements followed by the individual elements.
-  size_t cur_len= sprintf(buf, "%u", ids->elements);
+  size_t cur_len= sprintf(buf, "%zu", ids->elements);
   sz-= cur_len;
 
   for (uint i= 0; i < ids->elements; i++)
@@ -1971,15 +2011,16 @@ void prot_store_ids(THD *thd, DYNAMIC_ARRAY *ids)
         break the loop whenever remained space could not fit
         ellipses on the next cycle
       */
-      sprintf(dbuff + cur_len, "...");
+      cur_len+= sprintf(dbuff + cur_len, "...");
       break;
     }
-    cur_len += sprintf(buff + cur_len, "%s", dbuff);
+    cur_len+= sprintf(buff + cur_len, "%s", dbuff);
   }
-  thd->protocol->store(buff, &my_charset_bin);
+  thd->protocol->store(buff, cur_len, &my_charset_bin);
   return;
 }
 
+
 bool Master_info_index::flush_all_relay_logs()
 {
   DBUG_ENTER("flush_all_relay_logs");
