@@ -1190,6 +1190,7 @@ public:
     bool val_native(Native *to) override
     {
       DBUG_ASSERT(marked_for_read());
+      DBUG_ASSERT(!is_null());
       if (to->alloc(FbtImpl::binary_length()))
         return true;
       to->length(FbtImpl::binary_length());
@@ -1200,6 +1201,7 @@ public:
     Fbt to_fbt() const
     {
       DBUG_ASSERT(marked_for_read());
+      DBUG_ASSERT(!is_null());
       return Fbt::record_to_memory((const char*) ptr);
     }
 
@@ -1535,7 +1537,7 @@ public:
       str->append(type_handler_fbt()->name().lex_cstring());
       str->append(')');
     }
-    bool fix_length_and_dec() override
+    bool fix_length_and_dec(THD *thd) override
     {
       Type_std_attributes::operator=(Type_std_attributes_fbt());
       if (Fbt::fix_fields_maybe_null_on_conversion_to_fbt(args[0]))
@@ -1587,9 +1589,8 @@ public:
       if (!example)
         return false;
       value_cached= true;
-      null_value_inside= null_value=
-        example->val_native_with_conversion_result(current_thd,
-                                                   &m_value, type_handler());
+      null_value= example->val_native_with_conversion_result(current_thd,
+                                                 &m_value, type_handler());
       return true;
     }
     String* val_str(String *to)
