@@ -28,6 +28,7 @@
 #include "my_base.h"                   /* ha_rows, ha_key_alg */
 #include <mysql_com.h>                  /* USERNAME_LENGTH */
 #include "sql_bitmap.h"
+#include "lex_charset.h"
 
 struct TABLE;
 class Type_handler;
@@ -76,14 +77,14 @@ typedef struct st_key_part_info {	/* Info about a key part */
   uint  offset;                         /* Offset in record (from 0) */
   uint  null_offset;                    /* Offset to null_bit in record */
   /* Length of key part in bytes, excluding NULL flag and length bytes */
-  uint16 length;
+  uint length;
   /* 
     Number of bytes required to store the keypart value. This may be
     different from the "length" field as it also counts
      - possible NULL-flag byte (see HA_KEY_NULL_LENGTH)
      - possible HA_KEY_BLOB_LENGTH bytes needed to store actual value length.
   */
-  uint16 store_length;
+  uint store_length;
   uint16 key_type;
   field_index_t fieldnr;                /* Fieldnr begins counting from 1 */
   uint16 key_part_flag;                 /* 0 or HA_REVERSE_SORT */
@@ -537,7 +538,8 @@ public:
     OPT_OR_REPLACE= 16,                // CREATE OR REPLACE TABLE
     OPT_OR_REPLACE_SLAVE_GENERATED= 32,// REPLACE was added on slave, it was
                                        // not in the original query on master.
-    OPT_IF_EXISTS= 64
+    OPT_IF_EXISTS= 64,
+    OPT_CREATE_SELECT= 128             // CREATE ... SELECT
   };
 
 private:
@@ -565,6 +567,8 @@ public:
   { return m_options & OPT_OR_REPLACE_SLAVE_GENERATED; }
   bool like() const { return m_options & OPT_LIKE; }
   bool if_exists() const { return m_options & OPT_IF_EXISTS; }
+  bool is_create_select() const { return m_options & OPT_CREATE_SELECT; }
+
   void add(const DDL_options_st::Options other)
   {
     m_options= (Options) ((uint) m_options | (uint) other);
@@ -598,17 +602,84 @@ public:
 
 struct Lex_length_and_dec_st
 {
-private:
-  const char *m_length;
-  const char *m_dec;
+protected:
+  uint32 m_length;
+  uint8  m_dec;
+  uint8  m_collation_type:LEX_CHARSET_COLLATION_TYPE_BITS;
+  bool   m_has_explicit_length:1;
+  bool   m_has_explicit_dec:1;
+  bool   m_length_overflowed:1;
+  bool   m_dec_overflowed:1;
+
+  static_assert(LEX_CHARSET_COLLATION_TYPE_BITS <= 8,
+                "Lex_length_and_dec_st::m_collation_type bits check");
+
 public:
-  void set(const char *length, const char *dec)
+  void reset()
+  {
+    m_length= 0;
+    m_dec= 0;
+    m_collation_type= 0;
+    m_has_explicit_length= false;
+    m_has_explicit_dec= false;
+    m_length_overflowed= false;
+    m_dec_overflowed= false;
+  }
+  void set_length_only(uint32 length)
+  {
+    m_length= length;
+    m_dec= 0;
+    m_collation_type= 0;
+    m_has_explicit_length= true;
+    m_has_explicit_dec= false;
+    m_length_overflowed= false;
+    m_dec_overflowed= false;
+  }
+  void set_dec_only(uint8 dec)
+  {
+    m_length= 0;
+    m_dec= dec;
+    m_collation_type= 0;
+    m_has_explicit_length= false;
+    m_has_explicit_dec= true;
+    m_length_overflowed= false;
+    m_dec_overflowed= false;
+  }
+  void set_length_and_dec(uint32 length, uint8 dec)
   {
     m_length= length;
     m_dec= dec;
+    m_collation_type= 0;
+    m_has_explicit_length= true;
+    m_has_explicit_dec= true;
+    m_length_overflowed= false;
+    m_dec_overflowed= false;
+  }
+  void set(const char *length, const char *dec);
+  uint32 length() const
+  {
+    return m_length;
+  }
+  uint8 dec() const
+  {
+    return m_dec;
+  }
+  bool has_explicit_length() const
+  {
+    return m_has_explicit_length;
+  }
+  bool has_explicit_dec() const
+  {
+    return m_has_explicit_dec;
+  }
+  bool length_overflowed()  const
+  {
+    return m_length_overflowed;
+  }
+  bool dec_overflowed()  const
+  {
+    return m_dec_overflowed;
   }
-  const char *length() const { return m_length; }
-  const char *dec() const { return m_dec; }
 };
 
 
@@ -616,32 +687,59 @@ struct Lex_field_type_st: public Lex_length_and_dec_st
 {
 private:
   const Type_handler *m_handler;
-  void set(const Type_handler *handler, const char *length, const char *dec)
-  {
-    m_handler= handler;
-    Lex_length_and_dec_st::set(length, dec);
-  }
+  CHARSET_INFO *m_ci;
 public:
-  void set(const Type_handler *handler, Lex_length_and_dec_st length_and_dec)
+  void set(const Type_handler *handler,
+           Lex_length_and_dec_st length_and_dec,
+           CHARSET_INFO *cs= NULL)
   {
     m_handler= handler;
+    m_ci= cs;
     Lex_length_and_dec_st::operator=(length_and_dec);
   }
-  void set_handler_length_flags(const Type_handler *handler, const char *length,
-                                uint32 flags);
-  void set(const Type_handler *handler, const char *length)
+  void set(const Type_handler *handler,
+           const Lex_length_and_dec_st &length_and_dec,
+           const Lex_charset_collation_st &coll)
   {
-    set(handler, length, 0);
+    m_handler= handler;
+    m_ci= coll.charset_collation();
+    Lex_length_and_dec_st::operator=(length_and_dec);
+    m_collation_type= ((uint8) coll.type()) & 0x3;
   }
-  void set(const Type_handler *handler)
+  void set(const Type_handler *handler, const Lex_charset_collation_st &coll)
   {
-    set(handler, 0, 0);
+    m_handler= handler;
+    m_ci= coll.charset_collation();
+    Lex_length_and_dec_st::reset();
+    m_collation_type= ((uint8) coll.type()) & 0x3;
+  }
+  void set(const Type_handler *handler, CHARSET_INFO *cs= NULL)
+  {
+    m_handler= handler;
+    m_ci= cs;
+    Lex_length_and_dec_st::reset();
+  }
+  void set_handler_length_flags(const Type_handler *handler,
+                                const Lex_length_and_dec_st &length,
+                                uint32 flags);
+  void set_handler_length(const Type_handler *handler, uint32 length)
+  {
+    m_handler= handler;
+    m_ci= NULL;
+    Lex_length_and_dec_st::set_length_only(length);
   }
   void set_handler(const Type_handler *handler)
   {
     m_handler= handler;
   }
   const Type_handler *type_handler() const { return m_handler; }
+  CHARSET_INFO *charset_collation() const { return m_ci; }
+  Lex_charset_collation lex_charset_collation() const
+  {
+    return Lex_charset_collation(m_ci,
+                                 (Lex_charset_collation_st::Type)
+                                 m_collation_type);
+  }
 };
 
 
@@ -649,26 +747,38 @@ struct Lex_dyncol_type_st: public Lex_length_and_dec_st
 {
 private:
   int m_type; // enum_dynamic_column_type is not visible here, so use int
+  CHARSET_INFO *m_ci;
 public:
-  void set(int type, const char *length, const char *dec)
-  {
-    m_type= type;
-    Lex_length_and_dec_st::set(length, dec);
-  }
-  void set(int type, Lex_length_and_dec_st length_and_dec)
+  void set(int type, Lex_length_and_dec_st length_and_dec,
+           CHARSET_INFO *cs= NULL)
   {
     m_type= type;
+    m_ci= cs;
     Lex_length_and_dec_st::operator=(length_and_dec);
   }
-  void set(int type, const char *length)
-  {
-    set(type, length, 0);
-  }
   void set(int type)
   {
-    set(type, 0, 0);
+    m_type= type;
+    m_ci= NULL;
+    Lex_length_and_dec_st::reset();
+  }
+  void set(int type, CHARSET_INFO *cs)
+  {
+    m_type= type;
+    m_ci= cs;
+    Lex_length_and_dec_st::reset();
+  }
+  bool set(int type, const Lex_charset_collation_st &collation,
+           CHARSET_INFO *charset)
+  {
+    CHARSET_INFO *tmp= collation.resolved_to_character_set(charset);
+    if (!tmp)
+      return true;
+    set(type, tmp);
+    return false;
   }
   int dyncol_type() const { return m_type; }
+  CHARSET_INFO *charset_collation() const { return m_ci; }
 };
 
 
