@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 1996, 2016, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2016, 2020, MariaDB Corporation.
+Copyright (c) 2016, 2021, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -61,6 +61,7 @@ btr_pcur_reset(
 	cursor->btr_cur.index = NULL;
 	cursor->btr_cur.page_cur.rec = NULL;
 	cursor->old_rec = NULL;
+	cursor->old_n_core_fields = 0;
 	cursor->old_n_fields = 0;
 	cursor->old_stored = false;
 
@@ -111,8 +112,9 @@ btr_pcur_store_position(
 	page_cursor = btr_pcur_get_page_cur(cursor);
 
 	rec = page_cur_get_rec(page_cursor);
-	offs = rec - block->frame;
-	ut_ad(block->page.id().page_no() == page_get_page_no(block->frame));
+	offs = rec - block->page.frame;
+	ut_ad(block->page.id().page_no()
+	      == page_get_page_no(block->page.frame));
 	ut_ad(block->page.buf_fix_count());
 	/* For spatial index, when we do positioning on parent
 	buffer if necessary, it might not hold latches, but the
@@ -125,13 +127,13 @@ btr_pcur_store_position(
 
 	cursor->old_stored = true;
 
-	if (page_is_empty(block->frame)) {
+	if (page_is_empty(block->page.frame)) {
 		/* It must be an empty index tree; NOTE that in this case
 		we do not store the modify_clock, but always do a search
 		if we restore the cursor position */
 
-		ut_a(!page_has_siblings(block->frame));
-		ut_ad(page_is_leaf(block->frame));
+		ut_a(!page_has_siblings(block->page.frame));
+		ut_ad(page_is_leaf(block->page.frame));
 		ut_ad(block->page.id().page_no() == index->page);
 
 		if (page_rec_is_supremum_low(offs)) {
@@ -153,12 +155,14 @@ before_first:
 			/* If the table is emptied during an ALGORITHM=NOCOPY
 			DROP COLUMN ... that is not ALGORITHM=INSTANT,
 			then we must preserve any instant ADD metadata. */
-			ut_ad(index->table->instant);
+			ut_ad(index->table->instant
+			      || block->page.id().page_no() != index->page);
 #endif
-			ut_ad(index->is_instant());
-			ut_ad(page_get_n_recs(block->frame) == 1);
-			ut_ad(page_is_leaf(block->frame));
-			ut_ad(!page_has_prev(block->frame));
+			ut_ad(index->is_instant()
+			      || block->page.id().page_no() != index->page);
+			ut_ad(page_get_n_recs(block->page.frame) == 1);
+			ut_ad(page_is_leaf(block->page.frame));
+			ut_ad(!page_has_prev(block->page.frame));
 			cursor->rel_pos = BTR_PCUR_AFTER_LAST_IN_TREE;
 			return;
 		}
@@ -168,12 +172,9 @@ before_first:
 		rec = page_rec_get_next(rec);
 
 		if (rec_is_metadata(rec, *index)) {
-			ut_ad(!page_has_prev(block->frame));
-			ut_d(const rec_t* p = rec);
+			ut_ad(!page_has_prev(block->page.frame));
 			rec = page_rec_get_next(rec);
 			if (page_rec_is_supremum(rec)) {
-				ut_ad(page_has_next(block->frame)
-				      || rec_is_alter_metadata(p, *index));
 				goto before_first;
 			}
 		}
@@ -185,19 +186,21 @@ before_first:
 
 	if (index->is_ibuf()) {
 		ut_ad(!index->table->not_redundant());
-		cursor->old_n_fields = rec_get_n_fields_old(rec);
-	} else if (page_rec_is_leaf(rec)) {
-		cursor->old_n_fields = dict_index_get_n_unique_in_tree(index);
-	} else if (index->is_spatial()) {
-		ut_ad(dict_index_get_n_unique_in_tree_nonleaf(index)
-		      == DICT_INDEX_SPATIAL_NODEPTR_SIZE);
-		/* For R-tree, we have to compare
-		the child page numbers as well. */
-		cursor->old_n_fields = DICT_INDEX_SPATIAL_NODEPTR_SIZE + 1;
+		cursor->old_n_fields = uint16_t(rec_get_n_fields_old(rec));
 	} else {
-		cursor->old_n_fields = dict_index_get_n_unique_in_tree(index);
+		cursor->old_n_fields = static_cast<uint16>(
+			dict_index_get_n_unique_in_tree(index));
+		if (index->is_spatial() && !page_rec_is_leaf(rec)) {
+			ut_ad(dict_index_get_n_unique_in_tree_nonleaf(index)
+			      == DICT_INDEX_SPATIAL_NODEPTR_SIZE);
+			/* For R-tree, we have to compare
+			the child page numbers as well. */
+			cursor->old_n_fields
+				= DICT_INDEX_SPATIAL_NODEPTR_SIZE + 1;
+		}
 	}
 
+	cursor->old_n_core_fields = index->n_core_fields;
 	cursor->old_rec = rec_copy_prefix_to_buf(rec, index,
 						 cursor->old_n_fields,
 						 &cursor->old_rec_buf,
@@ -232,6 +235,7 @@ btr_pcur_copy_stored_position(
 			+ (pcur_donate->old_rec - pcur_donate->old_rec_buf);
 	}
 
+	pcur_receive->old_n_core_fields = pcur_donate->old_n_core_fields;
 	pcur_receive->old_n_fields = pcur_donate->old_n_fields;
 }
 
@@ -251,33 +255,33 @@ struct optimistic_latch_leaves
   {
     return hint && btr_cur_optimistic_latch_leaves(
              hint, cursor->modify_clock, latch_mode,
-             btr_pcur_get_btr_cur(cursor), __FILE__, __LINE__, mtr);
+             btr_pcur_get_btr_cur(cursor), mtr);
   }
 };
 
-/**************************************************************//**
-Restores the stored position of a persistent cursor bufferfixing the page and
-obtaining the specified latches. If the cursor position was saved when the
-(1) cursor was positioned on a user record: this function restores the position
-to the last record LESS OR EQUAL to the stored record;
-(2) cursor was positioned on a page infimum record: restores the position to
-the last record LESS than the user record which was the successor of the page
-infimum;
-(3) cursor was positioned on the page supremum: restores to the first record
-GREATER than the user record which was the predecessor of the supremum.
-(4) cursor was positioned before the first or after the last in an empty tree:
-restores to before first or after the last in the tree.
-@return TRUE if the cursor position was stored when it was on a user
-record and it can be restored on a user record whose ordering fields
-are identical to the ones of the original user record */
-ibool
-btr_pcur_restore_position_func(
-/*===========================*/
-	ulint		latch_mode,	/*!< in: BTR_SEARCH_LEAF, ... */
-	btr_pcur_t*	cursor,		/*!< in: detached persistent cursor */
-	const char*	file,		/*!< in: file name */
-	unsigned	line,		/*!< in: line where called */
-	mtr_t*		mtr)		/*!< in: mtr */
+/** Restores the stored position of a persistent cursor bufferfixing
+the page and obtaining the specified latches. If the cursor position
+was saved when the
+(1) cursor was positioned on a user record: this function restores the
+position to the last record LESS OR EQUAL to the stored record;
+(2) cursor was positioned on a page infimum record: restores the
+position to the last record LESS than the user record which was the
+successor of the page infimum;
+(3) cursor was positioned on the page supremum: restores to the first
+record GREATER than the user record which was the predecessor of the
+supremum.
+(4) cursor was positioned before the first or after the last in an
+empty tree: restores to before first or after the last in the tree.
+@param restore_latch_mode BTR_SEARCH_LEAF, ...
+@param mtr mtr
+@return btr_pcur_t::SAME_ALL cursor position on user rec and points on
+the record with the same field values as in the stored record,
+btr_pcur_t::SAME_UNIQ cursor position is on user rec and points on the
+record with the same unique field values as in the stored record,
+btr_pcur_t::NOT_SAME cursor position is not on user rec or points on
+the record with not the samebuniq field values as in the stored */
+btr_pcur_t::restore_status
+btr_pcur_t::restore_position(ulint restore_latch_mode, mtr_t *mtr)
 {
 	dict_index_t*	index;
 	dtuple_t*	tuple;
@@ -287,97 +291,97 @@ btr_pcur_restore_position_func(
 
 	ut_ad(mtr->is_active());
 	//ut_ad(cursor->old_stored);
-	ut_ad(cursor->pos_state == BTR_PCUR_WAS_POSITIONED
-	      || cursor->pos_state == BTR_PCUR_IS_POSITIONED);
+	ut_ad(pos_state == BTR_PCUR_WAS_POSITIONED
+	      || pos_state == BTR_PCUR_IS_POSITIONED);
 
-	index = btr_cur_get_index(btr_pcur_get_btr_cur(cursor));
+	index = btr_cur_get_index(&btr_cur);
 
 	if (UNIV_UNLIKELY
-	    (cursor->rel_pos == BTR_PCUR_AFTER_LAST_IN_TREE
-	     || cursor->rel_pos == BTR_PCUR_BEFORE_FIRST_IN_TREE)) {
+	    (rel_pos == BTR_PCUR_AFTER_LAST_IN_TREE
+	     || rel_pos == BTR_PCUR_BEFORE_FIRST_IN_TREE)) {
 		dberr_t err = DB_SUCCESS;
 
 		/* In these cases we do not try an optimistic restoration,
 		but always do a search */
 
 		err = btr_cur_open_at_index_side(
-			cursor->rel_pos == BTR_PCUR_BEFORE_FIRST_IN_TREE,
-			index, latch_mode,
-			btr_pcur_get_btr_cur(cursor), 0, mtr);
+			rel_pos == BTR_PCUR_BEFORE_FIRST_IN_TREE,
+			index, restore_latch_mode,
+			&btr_cur, 0, mtr);
 
 		if (err != DB_SUCCESS) {
 			ib::warn() << " Error code: " << err
-				   << " btr_pcur_restore_position_func "
-				   << " called from file: "
-				   << file << " line: " << line
+				   << " btr_pcur_t::restore_position "
 				   << " table: " << index->table->name
 				   << " index: " << index->name;
 		}
 
-		cursor->latch_mode =
-			BTR_LATCH_MODE_WITHOUT_INTENTION(latch_mode);
-		cursor->pos_state = BTR_PCUR_IS_POSITIONED;
-		cursor->block_when_stored.clear();
+		latch_mode =
+			BTR_LATCH_MODE_WITHOUT_INTENTION(restore_latch_mode);
+		pos_state = BTR_PCUR_IS_POSITIONED;
+		block_when_stored.clear();
 
-		return(FALSE);
+		return restore_status::NOT_SAME;
 	}
 
-	ut_a(cursor->old_rec);
-	ut_a(cursor->old_n_fields);
+	ut_a(old_rec);
+	ut_a(old_n_core_fields);
+	ut_a(old_n_core_fields <= index->n_core_fields);
+	ut_a(old_n_fields);
 
-	switch (latch_mode) {
+	switch (restore_latch_mode) {
 	case BTR_SEARCH_LEAF:
 	case BTR_MODIFY_LEAF:
 	case BTR_SEARCH_PREV:
 	case BTR_MODIFY_PREV:
 		/* Try optimistic restoration. */
 
-		if (cursor->block_when_stored.run_with_hint(
-			optimistic_latch_leaves(cursor, &latch_mode,
+		if (block_when_stored.run_with_hint(
+			optimistic_latch_leaves(this, &restore_latch_mode,
 						mtr))) {
-			cursor->pos_state = BTR_PCUR_IS_POSITIONED;
-			cursor->latch_mode = latch_mode;
+			pos_state = BTR_PCUR_IS_POSITIONED;
+			latch_mode = restore_latch_mode;
 
-			buf_block_dbg_add_level(
-				btr_pcur_get_block(cursor),
-				dict_index_is_ibuf(index)
-				? SYNC_IBUF_TREE_NODE : SYNC_TREE_NODE);
-
-			if (cursor->rel_pos == BTR_PCUR_ON) {
+			if (rel_pos == BTR_PCUR_ON) {
 #ifdef UNIV_DEBUG
 				const rec_t*	rec;
 				rec_offs	offsets1_[REC_OFFS_NORMAL_SIZE];
 				rec_offs	offsets2_[REC_OFFS_NORMAL_SIZE];
 				rec_offs*	offsets1 = offsets1_;
 				rec_offs*	offsets2 = offsets2_;
-				rec = btr_pcur_get_rec(cursor);
+				rec = btr_pcur_get_rec(this);
 
 				rec_offs_init(offsets1_);
 				rec_offs_init(offsets2_);
 
 				heap = mem_heap_create(256);
-				offsets1 = rec_get_offsets(
-					cursor->old_rec, index, offsets1, true,
-					cursor->old_n_fields, &heap);
-				offsets2 = rec_get_offsets(
-					rec, index, offsets2, true,
-					cursor->old_n_fields, &heap);
+				ut_ad(old_n_core_fields
+				      == index->n_core_fields);
 
-				ut_ad(!cmp_rec_rec(cursor->old_rec,
+				offsets1 = rec_get_offsets(
+					old_rec, index, offsets1,
+					old_n_core_fields,
+					old_n_fields, &heap);
+				offsets2 = rec_get_offsets(
+					rec, index, offsets2,
+					index->n_core_fields,
+					old_n_fields, &heap);
+
+				ut_ad(!cmp_rec_rec(old_rec,
 						   rec, offsets1, offsets2,
 						   index));
 				mem_heap_free(heap);
 #endif /* UNIV_DEBUG */
-				return(TRUE);
+				return restore_status::SAME_ALL;
 			}
 			/* This is the same record as stored,
 			may need to be adjusted for BTR_PCUR_BEFORE/AFTER,
 			depending on search mode and direction. */
-			if (btr_pcur_is_on_user_rec(cursor)) {
-				cursor->pos_state
+			if (btr_pcur_is_on_user_rec(this)) {
+				pos_state
 					= BTR_PCUR_IS_POSITIONED_OPTIMISTIC;
 			}
-			return(FALSE);
+			return restore_status::NOT_SAME;
 		}
 	}
 
@@ -385,13 +389,19 @@ btr_pcur_restore_position_func(
 
 	heap = mem_heap_create(256);
 
-	tuple = dict_index_build_data_tuple(cursor->old_rec, index, true,
-					    cursor->old_n_fields, heap);
+	tuple = dtuple_create(heap, old_n_fields);
+
+	dict_index_copy_types(tuple, index, old_n_fields);
+
+	rec_copy_prefix_to_dtuple(tuple, old_rec, index,
+				  old_n_core_fields,
+				  old_n_fields, heap);
+	ut_ad(dtuple_check_typed(tuple));
 
 	/* Save the old search mode of the cursor */
-	old_mode = cursor->search_mode;
+	old_mode = search_mode;
 
-	switch (cursor->rel_pos) {
+	switch (rel_pos) {
 	case BTR_PCUR_ON:
 		mode = PAGE_CUR_LE;
 		break;
@@ -406,40 +416,45 @@ btr_pcur_restore_position_func(
 		mode = PAGE_CUR_UNSUPP;
 	}
 
-	btr_pcur_open_with_no_init_func(index, tuple, mode, latch_mode,
-					cursor,
+	btr_pcur_open_with_no_init_func(index, tuple, mode, restore_latch_mode,
+					this,
 #ifdef BTR_CUR_HASH_ADAPT
 					NULL,
 #endif /* BTR_CUR_HASH_ADAPT */
-					file, line, mtr);
+					mtr);
 
 	/* Restore the old search mode */
-	cursor->search_mode = old_mode;
+	search_mode = old_mode;
 
-	ut_ad(cursor->rel_pos == BTR_PCUR_ON
-	      || cursor->rel_pos == BTR_PCUR_BEFORE
-	      || cursor->rel_pos == BTR_PCUR_AFTER);
+	ut_ad(rel_pos == BTR_PCUR_ON
+	      || rel_pos == BTR_PCUR_BEFORE
+	      || rel_pos == BTR_PCUR_AFTER);
 	rec_offs offsets[REC_OFFS_NORMAL_SIZE];
 	rec_offs_init(offsets);
-	if (cursor->rel_pos == BTR_PCUR_ON
-	    && btr_pcur_is_on_user_rec(cursor)
-	    && !cmp_dtuple_rec(tuple, btr_pcur_get_rec(cursor),
-			       rec_get_offsets(btr_pcur_get_rec(cursor),
-					       index, offsets, true,
-					       ULINT_UNDEFINED, &heap))) {
+	restore_status ret_val= restore_status::NOT_SAME;
+	if (rel_pos == BTR_PCUR_ON && btr_pcur_is_on_user_rec(this)) {
+		ulint n_matched_fields= 0;
+		if (!cmp_dtuple_rec_with_match(
+		      tuple, btr_pcur_get_rec(this), index,
+		      rec_get_offsets(btr_pcur_get_rec(this), index, offsets,
+			index->n_core_fields, ULINT_UNDEFINED, &heap),
+		      &n_matched_fields)) {
 
-		/* We have to store the NEW value for the modify clock,
-		since the cursor can now be on a different page!
-		But we can retain the value of old_rec */
+			/* We have to store the NEW value for the modify clock,
+			since the cursor can now be on a different page!
+			But we can retain the value of old_rec */
 
-		cursor->block_when_stored.store(btr_pcur_get_block(cursor));
-		cursor->modify_clock = buf_block_get_modify_clock(
-					cursor->block_when_stored.block());
-		cursor->old_stored = true;
+			block_when_stored.store(btr_pcur_get_block(this));
+			modify_clock= buf_block_get_modify_clock(
+			    block_when_stored.block());
+			old_stored= true;
 
-		mem_heap_free(heap);
+			mem_heap_free(heap);
 
-		return(TRUE);
+			return restore_status::SAME_ALL;
+		}
+		if (n_matched_fields >= index->n_uniq)
+			ret_val= restore_status::SAME_UNIQ;
 	}
 
 	mem_heap_free(heap);
@@ -448,9 +463,9 @@ btr_pcur_restore_position_func(
 	to the cursor because it can now be on a different page, the record
 	under it may have been removed, etc. */
 
-	btr_pcur_store_position(cursor, mtr);
+	btr_pcur_store_position(this, mtr);
 
-	return(FALSE);
+	return ret_val;
 }
 
 /*********************************************************//**
@@ -559,7 +574,7 @@ btr_pcur_move_backward_from_page(
 
 	mtr_start(mtr);
 
-	btr_pcur_restore_position(latch_mode2, cursor, mtr);
+	cursor->restore_position(latch_mode2, mtr);
 
 	page = btr_pcur_get_page(cursor);
 
@@ -632,8 +647,7 @@ in the first case sets the cursor after last in tree, and in the latter case
 before first in tree. The latching mode must be BTR_SEARCH_LEAF or
 BTR_MODIFY_LEAF. */
 void
-btr_pcur_open_on_user_rec_func(
-/*===========================*/
+btr_pcur_open_on_user_rec(
 	dict_index_t*	index,		/*!< in: index */
 	const dtuple_t*	tuple,		/*!< in: tuple on which search done */
 	page_cur_mode_t	mode,		/*!< in: PAGE_CUR_L, ... */
@@ -641,12 +655,9 @@ btr_pcur_open_on_user_rec_func(
 					BTR_MODIFY_LEAF */
 	btr_pcur_t*	cursor,		/*!< in: memory buffer for persistent
 					cursor */
-	const char*	file,		/*!< in: file name */
-	unsigned	line,		/*!< in: line where called */
 	mtr_t*		mtr)		/*!< in: mtr */
 {
-	btr_pcur_open_low(index, 0, tuple, mode, latch_mode, cursor,
-			  file, line, 0, mtr);
+	btr_pcur_open_low(index, 0, tuple, mode, latch_mode, cursor, 0, mtr);
 
 	if ((mode == PAGE_CUR_GE) || (mode == PAGE_CUR_G)) {
 
