@@ -316,16 +316,6 @@ dtuple_get_n_ext(
 /*=============*/
 	const dtuple_t*	tuple)	/*!< in: tuple */
 	MY_ATTRIBUTE((nonnull));
-/** Compare two data tuples.
-@param[in] tuple1 first data tuple
-@param[in] tuple2 second data tuple
-@return positive, 0, negative if tuple1 is greater, equal, less, than tuple2,
-respectively */
-int
-dtuple_coll_cmp(
-	const dtuple_t*	tuple1,
-	const dtuple_t*	tuple2)
-	MY_ATTRIBUTE((warn_unused_result));
 /** Fold a prefix given as the number of fields of a tuple.
 @param[in]	tuple		index record
 @param[in]	n_fields	number of complete fields to fold
@@ -544,6 +534,17 @@ struct dtuple_t {
 	@param[in]	index	index possibly with instantly added columns */
 	void trim(const dict_index_t& index);
 
+	bool vers_history_row() const
+	{
+		for (ulint i = 0; i < n_fields; i++) {
+			const dfield_t* field = &fields[i];
+			if (field->type.vers_sys_end()) {
+				return field->vers_history_row();
+			}
+		}
+		return false;
+	}
+
 	/**
 	@param info_bits	the info_bits of a data tuple
 	@return whether this is a hidden metadata record
@@ -694,6 +695,6 @@ struct big_rec_t {
 		ulint		n_fld);
 };
 
-#include "data0data.ic"
+#include "data0data.inl"
 
 #endif
