@@ -24,7 +24,8 @@ The low-level file system
 Created 10/25/1995 Heikki Tuuri
 *******************************************************/
 
-#pragma once
+#ifndef fil0fil_h
+#define fil0fil_h
 
 #include "fsp0types.h"
 #include "mach0data.h"
@@ -411,41 +412,30 @@ private:
   pthread_t latch_owner;
   ut_d(Atomic_relaxed<uint32_t> latch_count;)
 public:
-  /** MariaDB encryption data */
-  fil_space_crypt_t *crypt_data;
+	/** MariaDB encryption data */
+	fil_space_crypt_t* crypt_data;
 
-  /** Whether needs_flush(), or this is in fil_system.unflushed_spaces */
-  bool is_in_unflushed_spaces;
+	/** Checks that this tablespace in a list of unflushed tablespaces. */
+	bool is_in_unflushed_spaces;
 
-  /** Whether this in fil_system.default_encrypt_tables (needs key rotation) */
-  bool is_in_default_encrypt;
+	/** Checks that this tablespace needs key rotation. */
+	bool is_in_default_encrypt;
 
-private:
-  /** Whether any corrupton of this tablespace has been reported */
-  mutable std::atomic_flag is_corrupted;
+	/** mutex to protect freed ranges */
+	std::mutex	freed_range_mutex;
 
-public:
-  /** mutex to protect freed_ranges and last_freed_lsn */
-  std::mutex freed_range_mutex;
-private:
-  /** Ranges of freed page numbers; protected by freed_range_mutex */
-  range_set freed_ranges;
+	/** Variables to store freed ranges. This can be used to write
+	zeroes/punch the hole in files. Protected by freed_mutex */
+	range_set	freed_ranges;
 
-  /** LSN of freeing last page; protected by freed_range_mutex */
-  lsn_t last_freed_lsn;
+	/** Stores last page freed lsn. Protected by freed_mutex */
+	lsn_t		last_freed_lsn;
+
+	ulint		magic_n;/*!< FIL_SPACE_MAGIC_N */
 
-public:
   /** @return whether doublewrite buffering is needed */
   inline bool use_doublewrite() const;
 
-  /** @return whether a page has been freed */
-  inline bool is_freed(uint32_t page);
-
-  /** Apply freed_ranges to the file.
-  @param writable whether the file is writable
-  @return number of pages written or hole-punched */
-  uint32_t flush_freed(bool writable);
-
 	/** Append a file to the chain of files of a space.
 	@param[in]	name		file name of a file that is not open
 	@param[in]	handle		file handle, or OS_FILE_CLOSED
@@ -502,9 +492,6 @@ public:
   written while the space ID is being updated in each page. */
   inline void set_imported();
 
-  /** Report the tablespace as corrupted */
-  ATTRIBUTE_COLD void set_corrupted() const;
-
   /** @return whether the storage device is rotational (HDD, not SSD) */
   inline bool is_rotational() const;
 
@@ -536,16 +523,15 @@ public:
 
 private:
   MY_ATTRIBUTE((warn_unused_result))
-  /** Try to acquire a tablespace reference (increment referenced()).
-  @param avoid   when these flags are set, nothing will be acquired
-  @return the old reference count */
-  uint32_t acquire_low(uint32_t avoid= STOPPING)
+  /** Try to acquire a tablespace reference.
+  @return the old reference count (if STOPPING is set, it was not acquired) */
+  uint32_t acquire_low()
   {
     uint32_t n= 0;
     while (!n_pending.compare_exchange_strong(n, n + 1,
                                               std::memory_order_acquire,
                                               std::memory_order_relaxed) &&
-           !(n & avoid));
+           !(n & STOPPING));
     return n;
   }
 public:
@@ -559,8 +545,10 @@ public:
   @return whether the file is usable */
   bool acquire()
   {
-    const auto flags= acquire_low(STOPPING | CLOSING) & (STOPPING | CLOSING);
-    return UNIV_LIKELY(!flags) || (flags == CLOSING && acquire_and_prepare());
+    uint32_t n= acquire_low();
+    if (UNIV_LIKELY(!(n & (STOPPING | CLOSING))))
+      return true;
+    return UNIV_LIKELY(!(n & STOPPING)) && prepare();
   }
 
   /** Acquire another tablespace reference for I/O. */
@@ -646,7 +634,11 @@ public:
   /** @return last_freed_lsn */
   lsn_t get_last_freed_lsn() { return last_freed_lsn; }
   /** Update last_freed_lsn */
-  void update_last_freed_lsn(lsn_t lsn) { last_freed_lsn= lsn; }
+  void update_last_freed_lsn(lsn_t lsn)
+  {
+    std::lock_guard<std::mutex> freed_lock(freed_range_mutex);
+    last_freed_lsn= lsn;
+  }
 
   /** Note that the file will need fsync().
   @return whether this needs to be added to fil_system.unflushed_spaces */
@@ -667,7 +659,11 @@ public:
 
   /** Clear all freed ranges for undo tablespace when InnoDB
   encounters TRIM redo log record */
-  void clear_freed_ranges() { freed_ranges.clear(); }
+  void clear_freed_ranges()
+  {
+    std::lock_guard<std::mutex> freed_lock(freed_range_mutex);
+    freed_ranges.clear();
+  }
 #endif /* !UNIV_INNOCHECKSUM */
   /** FSP_SPACE_FLAGS and FSP_FLAGS_MEM_ flags;
   check fsp0types.h to more info about flags. */
@@ -940,6 +936,7 @@ public:
   /** Add the set of freed page ranges */
   void add_free_range(const range_t range)
   {
+    std::lock_guard<std::mutex> freed_lock(freed_range_mutex);
     freed_ranges.add_range(range);
   }
 
@@ -1030,13 +1027,14 @@ public:
 
 private:
   /** @return whether the file is usable for io() */
-  ATTRIBUTE_COLD bool prepare_acquired();
-  /** @return whether the file is usable for io() */
-  ATTRIBUTE_COLD bool acquire_and_prepare();
+  ATTRIBUTE_COLD bool prepare(bool have_mutex= false);
 #endif /*!UNIV_INNOCHECKSUM */
 };
 
 #ifndef UNIV_INNOCHECKSUM
+/** Value of fil_space_t::magic_n */
+#define	FIL_SPACE_MAGIC_N	89472
+
 /** File node of a tablespace or the log data space */
 struct fil_node_t final
 {
@@ -1217,9 +1215,8 @@ struct fil_addr_t {
 
 /** For the first page in a system tablespace data file(ibdata*, not *.ibd):
 the file has been flushed to disk at least up to this lsn
-For other pages of tablespaces not in innodb_checksum_algorithm=full_crc32
-format: 32-bit key version used to encrypt the page + 32-bit checksum
-or 64 bits of zero if no encryption */
+For other pages: 32-bit key version used to encrypt the page + 32-bit checksum
+or 64 bites of zero if no encryption */
 #define FIL_PAGE_FILE_FLUSH_LSN_OR_KEY_VERSION 26U
 
 /** This overloads FIL_PAGE_FILE_FLUSH_LSN for RTREE Split Sequence Number */
@@ -1393,7 +1390,7 @@ struct fil_system_t
 
 private:
   bool m_initialised;
-#ifdef __linux__
+#ifdef UNIV_LINUX
   /** available block devices that reside on non-rotational storage */
   std::vector<dev_t> ssd;
 public:
@@ -1778,4 +1775,7 @@ void test_make_filepath();
 @return	block size */
 ulint fil_space_get_block_size(const fil_space_t* space, unsigned offset);
 
+#include "fil0fil.inl"
 #endif /* UNIV_INNOCHECKSUM */
+
+#endif /* fil0fil_h */
