@@ -339,11 +339,27 @@ struct trx_lock_t
   /** lock wait start time */
   Atomic_relaxed<my_hrtime_t> suspend_time;
 
-#if  defined(UNIV_DEBUG) || !defined(DBUG_OFF)
   /** 2=high priority WSREP thread has marked this trx to abort;
   1=another transaction chose this as a victim in deadlock resolution. */
   Atomic_relaxed<byte> was_chosen_as_deadlock_victim;
 
+  /** Clear the deadlock victim status. */
+  void clear_deadlock_victim()
+  {
+#ifndef WITH_WSREP
+    was_chosen_as_deadlock_victim= false;
+#elif defined __GNUC__ && (defined __i386__ || defined __x86_64__)
+    /* There is no 8-bit version of the 80386 BTR instruction.
+    Technically, this is the wrong addressing mode (16-bit), but
+    there are other data members stored after the byte. */
+    __asm__ __volatile__("lock btrw $0, %0"
+                         : "+m" (was_chosen_as_deadlock_victim));
+#else
+    was_chosen_as_deadlock_victim.fetch_and(byte(~1));
+#endif
+  }
+
+#ifdef WITH_WSREP
   /** Flag the lock owner as a victim in Galera conflict resolution. */
   void set_wsrep_victim()
   {
@@ -357,17 +373,7 @@ struct trx_lock_t
     was_chosen_as_deadlock_victim.fetch_or(2);
 # endif
   }
-#else /* defined(UNIV_DEBUG) || !defined(DBUG_OFF) */
-
-  /** High priority WSREP thread has marked this trx to abort or
-  another transaction chose this as a victim in deadlock resolution. */
-  Atomic_relaxed<bool> was_chosen_as_deadlock_victim;
-
-  /** Flag the lock owner as a victim in Galera conflict resolution. */
-  void set_wsrep_victim() {
-    was_chosen_as_deadlock_victim= true;
-  }
-#endif /* defined(UNIV_DEBUG) || !defined(DBUG_OFF) */
+#endif
 
   /** Next available rec_pool[] entry */
   byte rec_cached;
@@ -433,8 +439,6 @@ class trx_mod_table_time_t
 
   /** Buffer to store insert opertion */
   row_merge_bulk_t *bulk_store= nullptr;
-
-  friend struct trx_t;
 public:
   /** Constructor
   @param rows   number of modified rows so far */
@@ -496,13 +500,6 @@ public:
     return false;
   }
 
-  /** @return the first undo record that modified the table */
-  undo_no_t get_first() const
-  {
-    ut_ad(valid());
-    return LIMIT & first;
-  }
-
   /** Add the tuple to the transaction bulk buffer for the given index.
   @param entry  tuple to be inserted
   @param index  bulk insert for the index
@@ -526,9 +523,11 @@ public:
   }
 
   /** @return whether the buffer storage exist */
-  bool bulk_buffer_exist() const
+  bool bulk_buffer_exist()
   {
-    return bulk_store && is_bulk_insert();
+    if (is_bulk_insert() && bulk_store)
+      return true;
+    return false;
   }
 };
 
@@ -617,20 +616,15 @@ struct trx_t : ilist_node<>
 {
 private:
   /**
-    Least significant 31 bits is count of references.
+    Count of references.
 
     We can't release the locks nor commit the transaction until this reference
     is 0. We can change the state to TRX_STATE_COMMITTED_IN_MEMORY to signify
     that it is no longer "active".
-
-    If the most significant bit is set this transaction should stop inheriting
-    (GAP)locks. Generally set to true during transaction prepare for RC or lower
-    isolation, if requested. Needed for replication replay where
-    we don't want to get blocked on GAP locks taken for protecting
-    concurrent unique insert or replace operation.
   */
+
   alignas(CPU_LEVEL1_DCACHE_LINESIZE)
-  Atomic_relaxed<uint32_t> skip_lock_inheritance_and_n_ref;
+  Atomic_counter<int32_t> n_ref;
 
 
 public:
@@ -1035,48 +1029,26 @@ public:
   void savepoints_discard(trx_named_savept_t *savept);
 
 
-  bool is_referenced() const
-  {
-    return (skip_lock_inheritance_and_n_ref & ~(1U << 31)) > 0;
-  }
+  bool is_referenced() const { return n_ref > 0; }
 
 
   void reference()
   {
-    ut_d(auto old_n_ref =)
-    skip_lock_inheritance_and_n_ref.fetch_add(1);
-    ut_ad(int32_t(old_n_ref << 1) >= 0);
+#ifdef UNIV_DEBUG
+    auto old_n_ref=
+#endif
+    n_ref++;
+    ut_ad(old_n_ref >= 0);
   }
 
+
   void release_reference()
   {
-    ut_d(auto old_n_ref =)
-    skip_lock_inheritance_and_n_ref.fetch_sub(1);
-    ut_ad(int32_t(old_n_ref << 1) > 0);
-  }
-
-  bool is_not_inheriting_locks() const
-  {
-    return skip_lock_inheritance_and_n_ref >> 31;
-  }
-
-  void set_skip_lock_inheritance()
-  {
-    ut_d(auto old_n_ref=) skip_lock_inheritance_and_n_ref.fetch_add(1U << 31);
-    ut_ad(!(old_n_ref >> 31));
-  }
-
-  void reset_skip_lock_inheritance()
-  {
-#if defined __GNUC__ && (defined __i386__ || defined __x86_64__)
-    __asm__("lock btrl $31, %0" : : "m"(skip_lock_inheritance_and_n_ref));
-#elif defined _MSC_VER && (defined _M_IX86 || defined _M_X64)
-    _interlockedbittestandreset(
-        reinterpret_cast<volatile long *>(&skip_lock_inheritance_and_n_ref),
-        31);
-#else
-    skip_lock_inheritance_and_n_ref.fetch_and(~1U << 31);
+#ifdef UNIV_DEBUG
+    auto old_n_ref=
 #endif
+    n_ref--;
+    ut_ad(old_n_ref > 0);
   }
 
   /** @return whether the table has lock on
@@ -1106,7 +1078,6 @@ public:
     ut_ad(UT_LIST_GET_LEN(lock.evicted_tables) == 0);
     ut_ad(!dict_operation);
     ut_ad(!apply_online_log);
-    ut_ad(!is_not_inheriting_locks());
   }
 
   /** This has to be invoked on SAVEPOINT or at the end of a statement.
@@ -1161,22 +1132,6 @@ public:
     return &it->second;
   }
 
-  /** Rollback all bulk insert operations */
-  void bulk_rollback()
-  {
-    undo_no_t low_limit= UINT64_MAX;
-    for (auto& t : mod_tables)
-    {
-      if (!t.second.is_bulk_insert())
-        continue;
-      if (t.second.get_first() < low_limit)
-        low_limit= t.second.get_first();
-    }
-
-    trx_savept_t bulk_save{low_limit};
-    rollback(&bulk_save);
-  }
-
   /** Do the bulk insert for the buffered insert operation
   for the transaction.
   @return DB_SUCCESS or error code */
@@ -1189,10 +1144,7 @@ public:
     for (auto& t : mod_tables)
       if (t.second.is_bulk_insert())
         if (dberr_t err= t.second.write_bulk(t.first, this))
-        {
-          bulk_rollback();
           return err;
-        }
     return DB_SUCCESS;
   }
 
