@@ -296,6 +296,50 @@ struct ReleaseAll {
   }
 };
 
+/** Stops iteration is savepoint is reached */
+template <typename Functor> struct TillSavepoint
+{
+
+  /** Constructor
+  @param[in] functor functor which is called if savepoint is not reached
+  @param[in] savepoint savepoint value to rollback
+  @param[in] used current position in slots container */
+  TillSavepoint(const Functor &functor, ulint savepoint, ulint used)
+      : functor(functor),
+        m_slots_count((used - savepoint) / sizeof(mtr_memo_slot_t))
+  {
+    ut_ad(savepoint);
+    ut_ad(used >= savepoint);
+  }
+
+  /** @return true if savepoint is not reached, false otherwise */
+  bool operator()(mtr_memo_slot_t *slot)
+  {
+#ifdef UNIV_DEBUG
+    /** This check is added because the code is invoked only from
+    row_search_mvcc() to release latches acquired during clustered index search
+    for secondary index record. To make it more universal we could add one more
+    member in this functor for debug build to pass only certain slot types,
+    but this is currently not necessary. */
+    switch (slot->type)
+    {
+    case MTR_MEMO_S_LOCK:
+    case MTR_MEMO_PAGE_S_FIX:
+      break;
+    default:
+      ut_a(false);
+    }
+#endif
+    return m_slots_count-- && functor(slot);
+  }
+
+private:
+  /** functor to invoke */
+  const Functor &functor;
+  /** slots count left till savepoint */
+  ulint m_slots_count;
+};
+
 #ifdef UNIV_DEBUG
 /** Check that all slots have been handled. */
 struct DebugCheck {
@@ -312,16 +356,10 @@ struct DebugCheck {
 struct ReleaseBlocks
 {
   const lsn_t start, end;
-#ifdef UNIV_DEBUG
-  const mtr_buf_t &memo;
-
-  ReleaseBlocks(lsn_t start, lsn_t end, const mtr_buf_t &memo) :
-    start(start), end(end), memo(memo)
-#else /* UNIV_DEBUG */
-  ReleaseBlocks(lsn_t start, lsn_t end, const mtr_buf_t&) :
-    start(start), end(end)
-#endif /* UNIV_DEBUG */
+  mutable size_t modified;
+  ReleaseBlocks(lsn_t start, lsn_t end) : start(start), end(end), modified(0)
   {
+    ut_ad(!srv_read_only_mode);
     ut_ad(start);
     ut_ad(end);
   }
@@ -340,8 +378,25 @@ struct ReleaseBlocks
       return true;
     }
 
-    buf_flush_note_modification(static_cast<buf_block_t*>(slot->object),
-                                start, end);
+    modified++;
+    buf_block_t *b= static_cast<buf_block_t*>(slot->object);
+    ut_d(const auto s= b->page.state());
+    ut_ad(s > buf_page_t::FREED);
+    ut_ad(s < buf_page_t::READ_FIX);
+    ut_ad(mach_read_from_8(b->page.frame + FIL_PAGE_LSN) <= end);
+    mach_write_to_8(b->page.frame + FIL_PAGE_LSN, end);
+    if (UNIV_LIKELY_NULL(b->page.zip.data))
+      memcpy_aligned<8>(FIL_PAGE_LSN + b->page.zip.data,
+                        FIL_PAGE_LSN + b->page.frame, 8);
+
+    const lsn_t oldest_modification= b->page.oldest_modification();
+
+    if (oldest_modification > 1)
+      ut_ad(oldest_modification <= start);
+    else if (fsp_is_system_temporary(b->page.id().space()))
+      b->page.set_temp_modified();
+    else
+      buf_pool.insert_into_flush_list(b, start);
     return true;
   }
 };
@@ -366,6 +421,7 @@ void mtr_t::start()
   new(&m_log) mtr_buf_t();
 
   m_made_dirty= false;
+  m_latch_ex= false;
   m_inside_ibuf= false;
   m_modifications= false;
   m_log_mode= MTR_LOG_ALL;
@@ -394,6 +450,7 @@ void mtr_t::commit()
   /* This is a dirty read, for debugging. */
   ut_ad(!m_modifications || !recv_no_log_write);
   ut_ad(!m_modifications || m_log_mode != MTR_LOG_NONE);
+  ut_ad(!m_latch_ex);
 
   if (m_modifications && (m_log_mode == MTR_LOG_NO_REDO || !m_log.empty()))
   {
@@ -404,14 +461,14 @@ void mtr_t::commit()
     if (UNIV_LIKELY(m_log_mode == MTR_LOG_ALL))
     {
       lsns= do_write();
-
-      if (m_made_dirty)
-        mysql_mutex_lock(&log_sys.flush_order_mutex);
-
-      /* It is now safe to release log_sys.mutex because the
-      buf_pool.flush_order_mutex will ensure that we are the first one
-      to insert into buf_pool.flush_list. */
-      mysql_mutex_unlock(&log_sys.mutex);
+      if (m_made_dirty);
+      else if (m_latch_ex)
+      {
+        log_sys.latch.wr_unlock();
+        m_latch_ex= false;
+      }
+      else
+        log_sys.latch.rd_unlock();
     }
     else
     {
@@ -420,7 +477,7 @@ void mtr_t::commit()
       m_commit_lsn= log_sys.get_lsn();
       lsns= { m_commit_lsn, PAGE_FLUSH_NO };
       if (UNIV_UNLIKELY(m_made_dirty)) /* This should be IMPORT TABLESPACE */
-        mysql_mutex_lock(&log_sys.flush_order_mutex);
+        log_sys.latch.rd_lock(SRW_LOCK_CALL);
     }
 
     if (m_freed_pages)
@@ -445,16 +502,29 @@ void mtr_t::commit()
     else
       ut_ad(!m_freed_space);
 
-    m_memo.for_each_block_in_reverse(CIterate<const ReleaseBlocks>
-                                     (ReleaseBlocks(lsns.first, m_commit_lsn,
-                                                    m_memo)));
-    if (m_made_dirty)
-      mysql_mutex_unlock(&log_sys.flush_order_mutex);
+    Iterate<ReleaseBlocks> rb{ReleaseBlocks{lsns.first, m_commit_lsn}};
+    m_memo.for_each_block_in_reverse(rb);
+    if (!m_made_dirty);
+    else if (m_latch_ex)
+    {
+      log_sys.latch.wr_unlock();
+      m_latch_ex= false;
+    }
+    else
+      log_sys.latch.rd_unlock();
 
     m_memo.for_each_block_in_reverse(CIterate<ReleaseLatches>());
 
     if (UNIV_UNLIKELY(lsns.second != PAGE_FLUSH_NO))
       buf_flush_ahead(m_commit_lsn, lsns.second == PAGE_FLUSH_SYNC);
+
+    if (rb.functor.modified)
+    {
+      mysql_mutex_lock(&buf_pool.flush_list_mutex);
+      buf_pool.flush_list_requests+= rb.functor.modified;
+      buf_pool.page_cleaner_wakeup();
+      mysql_mutex_unlock(&buf_pool.flush_list_mutex);
+    }
   }
   else
     m_memo.for_each_block_in_reverse(CIterate<ReleaseAll>());
@@ -462,6 +532,21 @@ void mtr_t::commit()
   release_resources();
 }
 
+/** Release latches till savepoint. To simplify the code only
+MTR_MEMO_S_LOCK and MTR_MEMO_PAGE_S_FIX slot types are allowed to be
+released, otherwise it would be neccesary to add one more argument in the
+function to point out what slot types are allowed for rollback, and this
+would be overengineering as corrently the function is used only in one place
+in the code.
+@param savepoint   savepoint, can be obtained with get_savepoint */
+void mtr_t::rollback_to_savepoint(ulint savepoint)
+{
+  Iterate<TillSavepoint<ReleaseLatches>> iteration(
+      TillSavepoint<ReleaseLatches>(ReleaseLatches(), savepoint,
+                                    get_savepoint()));
+  m_memo.for_each_block_in_reverse(iteration);
+}
+
 /** Shrink a tablespace. */
 struct Shrink
 {
@@ -523,12 +608,16 @@ void mtr_t::commit_shrink(fil_space_t &space)
   ut_ad(UT_LIST_GET_LEN(space.chain) == 1);
 
   log_write_and_flush_prepare();
+  m_latch_ex= true;
+  log_sys.latch.wr_lock(SRW_LOCK_CALL);
 
   const lsn_t start_lsn= do_write().first;
 
-  mysql_mutex_lock(&log_sys.flush_order_mutex);
   /* Durably write the reduced FSP_SIZE before truncating the data file. */
   log_write_and_flush();
+#ifndef SUX_LOCK_GENERIC
+  ut_ad(log_sys.latch.is_write_locked());
+#endif
 
   os_file_truncate(space.chain.start->name, space.chain.start->handle,
                    os_offset_t{space.size} << srv_page_size_shift, true);
@@ -557,9 +646,9 @@ void mtr_t::commit_shrink(fil_space_t &space)
   m_memo.for_each_block_in_reverse(CIterate<Shrink>{space});
 
   m_memo.for_each_block_in_reverse(CIterate<const ReleaseBlocks>
-                                   (ReleaseBlocks(start_lsn, m_commit_lsn,
-                                                  m_memo)));
-  mysql_mutex_unlock(&log_sys.flush_order_mutex);
+                                   (ReleaseBlocks{start_lsn, m_commit_lsn}));
+  log_sys.latch.wr_unlock();
+  m_latch_ex= false;
 
   mysql_mutex_lock(&fil_system.mutex);
   ut_ad(space.is_being_truncated);
@@ -582,7 +671,9 @@ This is to be used at log_checkpoint().
 @return current LSN */
 lsn_t mtr_t::commit_files(lsn_t checkpoint_lsn)
 {
-  mysql_mutex_assert_owner(&log_sys.mutex);
+#ifndef SUX_LOCK_GENERIC
+  ut_ad(log_sys.latch.is_write_locked());
+#endif
   ut_ad(is_active());
   ut_ad(!is_inside_ibuf());
   ut_ad(m_log_mode == MTR_LOG_ALL);
@@ -592,6 +683,9 @@ lsn_t mtr_t::commit_files(lsn_t checkpoint_lsn)
   ut_ad(!m_freed_space);
   ut_ad(!m_freed_pages);
   ut_ad(!m_user_space);
+  ut_ad(!m_latch_ex);
+
+  m_latch_ex= true;
 
   if (checkpoint_lsn)
   {
@@ -753,8 +847,6 @@ mtr_t::release_page(const void* ptr, mtr_memo_type_t type)
 	ut_ad(0);
 }
 
-static bool log_margin_warned;
-static time_t log_margin_warn_time;
 static bool log_close_warned;
 static time_t log_close_warn_time;
 
@@ -774,65 +866,75 @@ ATTRIBUTE_COLD static void log_overwrite_warning(lsn_t age, lsn_t capacity)
   }
 }
 
-/** Reserve space in the log buffer for appending data.
-@param size   upper limit of the length of the data to append(), in bytes
-@return the current LSN */
-inline lsn_t log_t::append_prepare(size_t size) noexcept
+/** Wait in append_prepare() for buffer to become available
+@param ex   whether log_sys.latch is exclusively locked */
+ATTRIBUTE_COLD void log_t::append_prepare_wait(bool ex) noexcept
 {
-  mysql_mutex_assert_owner(&mutex);
+  log_sys.waits++;
+  log_sys.unlock_lsn();
 
-  lsn_t lsn= get_lsn();
+  if (ex)
+    log_sys.latch.wr_unlock();
+  else
+    log_sys.latch.rd_unlock();
 
-  if (UNIV_UNLIKELY(size > log_capacity))
+  DEBUG_SYNC_C("log_buf_size_exceeded");
+  log_buffer_flush_to_disk(log_sys.is_pmem());
+
+  if (ex)
+    log_sys.latch.wr_lock(SRW_LOCK_CALL);
+  else
+    log_sys.latch.rd_lock(SRW_LOCK_CALL);
+
+  log_sys.lock_lsn();
+}
+
+/** Reserve space in the log buffer for appending data.
+@tparam pmem  log_sys.is_pmem()
+@param size   total length of the data to append(), in bytes
+@param ex     whether log_sys.latch is exclusively locked
+@return the start LSN and the buffer position for append() */
+template<bool pmem>
+inline
+std::pair<lsn_t,byte*> log_t::append_prepare(size_t size, bool ex) noexcept
+{
+#ifndef SUX_LOCK_GENERIC
+  ut_ad(latch.is_locked());
+# ifndef _WIN32 // there is no accurate is_write_locked() on SRWLOCK
+  ut_ad(ex == latch.is_write_locked());
+# endif
+#endif
+  ut_ad(pmem == is_pmem());
+  const lsn_t checkpoint_margin{last_checkpoint_lsn + log_capacity - size};
+  const size_t avail{(pmem ? size_t(capacity()) : buf_size) - size};
+  lock_lsn();
+  write_to_buf++;
+
+  for (ut_d(int count= 50);
+       UNIV_UNLIKELY((pmem
+                      ? size_t(get_lsn() -
+                               get_flushed_lsn(std::memory_order_relaxed))
+                      : size_t{buf_free}) > avail); )
   {
-    time_t t= time(nullptr);
-
-    /* return with warning output to avoid deadlock */
-    if (!log_margin_warned || difftime(t, log_margin_warn_time) > 15)
-    {
-      log_margin_warned= true;
-      log_margin_warn_time= t;
-
-      sql_print_error("InnoDB: innodb_log_file_size is too small "
-                      "for mini-transaction size %zu", size);
-    }
-    goto throttle;
+    append_prepare_wait(ex);
+    ut_ad(count--);
   }
-  else if (UNIV_UNLIKELY(lsn + size > last_checkpoint_lsn + log_capacity))
-  throttle:
+
+  const lsn_t l{lsn.load(std::memory_order_relaxed)};
+  lsn.store(l + size, std::memory_order_relaxed);
+  const size_t b{buf_free};
+  size_t new_buf_free{b};
+  new_buf_free+= size;
+  if (pmem && new_buf_free >= file_size)
+    new_buf_free-= size_t(capacity());
+  buf_free= new_buf_free;
+  unlock_lsn();
+
+  if (UNIV_UNLIKELY(l > checkpoint_margin) ||
+      (!pmem && b >= max_buf_free))
     set_check_flush_or_checkpoint();
 
-  if (is_pmem())
-  {
-    for (ut_d(int count= 50); capacity() - size <
-           size_t(lsn - flushed_to_disk_lsn.load(std::memory_order_relaxed)); )
-    {
-      waits++;
-      mysql_mutex_unlock(&mutex);
-      DEBUG_SYNC_C("log_buf_size_exceeded");
-      log_write_up_to(lsn, true);
-      ut_ad(count--);
-      mysql_mutex_lock(&mutex);
-      lsn= get_lsn();
-    }
-    return lsn;
-  }
-
-  /* Calculate the amount of free space needed. */
-  size= (4 * 4096) - size + log_sys.buf_size;
-
-  for (ut_d(int count= 50); UNIV_UNLIKELY(buf_free > size); )
-  {
-    waits++;
-    mysql_mutex_unlock(&mutex);
-    DEBUG_SYNC_C("log_buf_size_exceeded");
-    log_write_up_to(lsn, false);
-    ut_ad(count--);
-    mysql_mutex_lock(&mutex);
-    lsn= get_lsn();
-  }
-
-  return lsn;
+  return {l, &buf[b]};
 }
 
 /** Finish appending data to the log.
@@ -840,9 +942,9 @@ inline lsn_t log_t::append_prepare(size_t size) noexcept
 @return whether buf_flush_ahead() will have to be invoked */
 static mtr_t::page_flush_ahead log_close(lsn_t lsn) noexcept
 {
-  mysql_mutex_assert_owner(&log_sys.mutex);
-  log_sys.write_to_buf++;
-  log_sys.set_lsn(lsn);
+#ifndef SUX_LOCK_GENERIC
+  ut_ad(log_sys.latch.is_locked());
+#endif
 
   const lsn_t checkpoint_age= lsn - log_sys.last_checkpoint_lsn;
 
@@ -863,6 +965,9 @@ std::pair<lsn_t,mtr_t::page_flush_ahead> mtr_t::do_write()
 {
   ut_ad(!recv_no_log_write);
   ut_ad(m_log_mode == MTR_LOG_ALL);
+#ifndef SUX_LOCK_GENERIC
+  ut_ad(!m_latch_ex || log_sys.latch.is_write_locked());
+#endif
 
   size_t len= m_log.size() + 5;
   ut_ad(len > 5);
@@ -879,86 +984,204 @@ std::pair<lsn_t,mtr_t::page_flush_ahead> mtr_t::do_write()
     { m_crc= my_crc32c(m_crc, b->begin(), b->used()); return true; });
   }
 
-  mysql_mutex_lock(&log_sys.mutex);
+  if (!m_latch_ex)
+    log_sys.latch.rd_lock(SRW_LOCK_CALL);
 
-  if (m_user_space && !is_predefined_tablespace(m_user_space->id) &&
-      !m_user_space->max_lsn)
+  if (UNIV_UNLIKELY(m_user_space && !m_user_space->max_lsn &&
+                    !is_predefined_tablespace(m_user_space->id)))
+  {
+    if (!m_latch_ex)
+    {
+      m_latch_ex= true;
+      log_sys.latch.rd_unlock();
+      log_sys.latch.wr_lock(SRW_LOCK_CALL);
+      if (UNIV_UNLIKELY(m_user_space->max_lsn != 0))
+        goto func_exit;
+    }
     name_write();
-
+  }
+func_exit:
   return finish_write(len);
 }
 
+inline void log_t::resize_write(lsn_t lsn, const byte *end, size_t len,
+                                size_t seq) noexcept
+{
+#ifndef SUX_LOCK_GENERIC
+  ut_ad(latch.is_locked());
+#endif
+
+  if (UNIV_LIKELY_NULL(resize_buf))
+  {
+    ut_ad(end >= buf);
+    end-= len;
+    size_t s;
+
+#ifdef HAVE_PMEM
+    if (!resize_flush_buf)
+    {
+      ut_ad(is_pmem());
+      const size_t resize_capacity{resize_target - START_OFFSET};
+      const lsn_t resizing{resize_in_progress()};
+      if (UNIV_UNLIKELY(lsn < resizing))
+      {
+        size_t l= resizing - lsn;
+        if (l >= len)
+          return;
+        end+= l - len;
+        len-= l;
+        lsn+= l;
+      }
+      lsn-= resizing;
+      s= START_OFFSET + lsn % resize_capacity;
+
+      if (UNIV_UNLIKELY(end < &buf[START_OFFSET]))
+      {
+        /* The source buffer (log_sys.buf) wrapped around */
+        ut_ad(end + capacity() < &buf[file_size]);
+        ut_ad(end + len >= &buf[START_OFFSET]);
+        ut_ad(end + capacity() + len >= &buf[file_size]);
+
+        size_t l= size_t(buf - (end - START_OFFSET));
+        if (UNIV_LIKELY(s + len <= resize_target))
+        {
+          /* The destination buffer (log_sys.resize_buf) did not wrap around */
+          memcpy(resize_buf + s, end + capacity(), l);
+          memcpy(resize_buf + s + l, &buf[START_OFFSET], len - l);
+          goto pmem_nowrap;
+        }
+        else
+        {
+          /* Both log_sys.buf and log_sys.resize_buf wrapped around */
+          const size_t rl= resize_target - s;
+          if (l <= rl)
+          {
+            /* log_sys.buf wraps around first */
+            memcpy(resize_buf + s, end + capacity(), l);
+            memcpy(resize_buf + s + l, &buf[START_OFFSET], rl - l);
+            memcpy(resize_buf + START_OFFSET, &buf[START_OFFSET + rl - l],
+                   len - l);
+          }
+          else
+          {
+            /* log_sys.resize_buf wraps around first */
+            memcpy(resize_buf + s, end + capacity(), rl);
+            memcpy(resize_buf + START_OFFSET, end + capacity() + rl, l - rl);
+            memcpy(resize_buf + START_OFFSET + (l - rl),
+                   &buf[START_OFFSET], len - l);
+          }
+          goto pmem_wrap;
+        }
+      }
+      else
+      {
+        ut_ad(end + len <= &buf[file_size]);
+
+        if (UNIV_LIKELY(s + len <= resize_target))
+        {
+          memcpy(resize_buf + s, end, len);
+        pmem_nowrap:
+          s+= len - seq;
+        }
+        else
+        {
+          /* The log_sys.resize_buf wrapped around */
+          memcpy(resize_buf + s, end, resize_target - s);
+          memcpy(resize_buf + START_OFFSET, end + (resize_target - s),
+                 len - (resize_target - s));
+        pmem_wrap:
+          s+= len - seq;
+          if (s >= resize_target)
+            s-= resize_capacity;
+          resize_lsn.fetch_add(resize_capacity); /* Move the target ahead. */
+        }
+      }
+    }
+    else
+#endif
+    {
+      ut_ad(resize_flush_buf);
+      s= end - buf;
+      ut_ad(s + len <= buf_size);
+      memcpy(resize_buf + s, end, len);
+      s+= len - seq;
+    }
+
+    /* Always set the sequence bit. If the resized log were to wrap around,
+    we will advance resize_lsn. */
+    ut_ad(resize_buf[s] <= 1);
+    resize_buf[s]= 1;
+  }
+}
+
 /** Write the mini-transaction log to the redo log buffer.
+@param len   number of bytes to write
 @return {start_lsn,flush_ahead} */
-std::pair<lsn_t,mtr_t::page_flush_ahead> mtr_t::finish_write(size_t len)
+std::pair<lsn_t,mtr_t::page_flush_ahead>
+mtr_t::finish_write(size_t len)
 {
   ut_ad(!recv_no_log_write);
   ut_ad(m_log_mode == MTR_LOG_ALL);
+#ifndef SUX_LOCK_GENERIC
+# ifndef _WIN32 // there is no accurate is_write_locked() on SRWLOCK
+  ut_ad(m_latch_ex == log_sys.latch.is_write_locked());
+# endif
+#endif
 
-  const lsn_t start_lsn= log_sys.append_prepare(len);
   const size_t size{m_commit_lsn ? 5U + 8U : 5U};
+  std::pair<lsn_t, byte*> start;
 
   if (!log_sys.is_pmem())
   {
-    m_log.for_each_block([](const mtr_buf_t::block_t *b)
-    { log_sys.append(b->begin(), b->used()); return true; });
-
-    if (log_sys.buf_free >= log_sys.max_buf_free)
-      log_sys.set_check_flush_or_checkpoint();
+    start= log_sys.append_prepare<false>(len, m_latch_ex);
+    m_log.for_each_block([&start](const mtr_buf_t::block_t *b)
+    { log_sys.append(start.second, b->begin(), b->used()); return true; });
 
 #ifdef HAVE_PMEM
   write_trailer:
 #endif
-    log_sys.buf[log_sys.buf_free]=
-      log_sys.get_sequence_bit(start_lsn + len - size);
+    *start.second++= log_sys.get_sequence_bit(start.first + len - size);
     if (m_commit_lsn)
     {
-      byte *nonce= log_sys.buf + log_sys.buf_free + 1;
-      mach_write_to_8(nonce, m_commit_lsn);
-      m_crc= my_crc32c(m_crc, nonce, 8);
-      mach_write_to_4(&log_sys.buf[log_sys.buf_free + 9], m_crc);
-      log_sys.buf_free+= 8 + 5;
-    }
-    else
-    {
-      mach_write_to_4(&log_sys.buf[log_sys.buf_free + 1], m_crc);
-      log_sys.buf_free+= 5;
+      mach_write_to_8(start.second, m_commit_lsn);
+      m_crc= my_crc32c(m_crc, start.second, 8);
+      start.second+= 8;
     }
+    mach_write_to_4(start.second, m_crc);
+    start.second+= 4;
   }
 #ifdef HAVE_PMEM
-  else if (UNIV_LIKELY(log_sys.buf_free + len < log_sys.file_size))
-  {
-    m_log.for_each_block([](const mtr_buf_t::block_t *b)
-    { log_sys.append(b->begin(), b->used()); return true; });
-    goto write_trailer;
-  }
   else
   {
-    m_log.for_each_block([](const mtr_buf_t::block_t *b)
+    start= log_sys.append_prepare<true>(len, m_latch_ex);
+    if (UNIV_LIKELY(start.second + len <= &log_sys.buf[log_sys.file_size]))
+    {
+      m_log.for_each_block([&start](const mtr_buf_t::block_t *b)
+      { log_sys.append(start.second, b->begin(), b->used()); return true; });
+      goto write_trailer;
+    }
+    m_log.for_each_block([&start](const mtr_buf_t::block_t *b)
     {
       size_t size{b->used()};
-      const size_t size_left{log_sys.file_size - log_sys.buf_free};
+      const size_t size_left(&log_sys.buf[log_sys.file_size] - start.second);
       const byte *src= b->begin();
-      if (size <= size_left)
-      {
-        ::memcpy(log_sys.buf + log_sys.buf_free, src, size);
-        log_sys.buf_free+= size;
-      }
-      else
+      if (size > size_left)
       {
+        ::memcpy(start.second, src, size_left);
+        start.second= &log_sys.buf[log_sys.START_OFFSET];
+        src+= size_left;
         size-= size_left;
-        ::memcpy(log_sys.buf + log_sys.buf_free, src, size_left);
-        ::memcpy(log_sys.buf + log_sys.START_OFFSET, src + size_left, size);
-        log_sys.buf_free= log_sys.START_OFFSET + size;
       }
+      ::memcpy(start.second, src, size);
+      start.second+= size;
       return true;
     });
-    const size_t size_left{log_sys.file_size - log_sys.buf_free};
+    const size_t size_left(&log_sys.buf[log_sys.file_size] - start.second);
     if (size_left > size)
       goto write_trailer;
 
     byte tail[5 + 8];
-    tail[0]= log_sys.get_sequence_bit(start_lsn + len - size);
+    tail[0]= log_sys.get_sequence_bit(start.first + len - size);
 
     if (m_commit_lsn)
     {
@@ -969,15 +1192,19 @@ std::pair<lsn_t,mtr_t::page_flush_ahead> mtr_t::finish_write(size_t len)
     else
       mach_write_to_4(tail + 1, m_crc);
 
-    ::memcpy(log_sys.buf + log_sys.buf_free, tail, size_left);
+    ::memcpy(start.second, tail, size_left);
     ::memcpy(log_sys.buf + log_sys.START_OFFSET, tail + size_left,
              size - size_left);
-    log_sys.buf_free= log_sys.START_OFFSET + (size - size_left);
+    start.second= log_sys.buf +
+      ((size >= size_left) ? log_sys.START_OFFSET : log_sys.file_size) +
+      (size - size_left);
   }
 #endif
 
-  m_commit_lsn= start_lsn + len;
-  return {start_lsn, log_close(m_commit_lsn)};
+  log_sys.resize_write(start.first, start.second, len, size);
+
+  m_commit_lsn= start.first + len;
+  return {start.first, log_close(m_commit_lsn)};
 }
 
 /** Find out whether a block was not X-latched by the mini-transaction */
