@@ -514,7 +514,7 @@ static srv_sys_t	srv_sys;
 struct purge_coordinator_state
 {
   /** Snapshot of the last history length before the purge call.*/
-  size_t m_history_length;
+  uint32 m_history_length;
   Atomic_counter<int> m_running;
 private:
   ulint count;
@@ -1068,7 +1068,7 @@ srv_export_innodb_status(void)
 		- UT_LIST_GET_LEN(buf_pool.free);
 
 	export_vars.innodb_max_trx_id = trx_sys.get_max_trx_id();
-	export_vars.innodb_history_list_length = trx_sys.history_size_approx();
+	export_vars.innodb_history_list_length = trx_sys.history_size();
 
 	mysql_mutex_lock(&lock_sys.wait_mutex);
 	export_vars.innodb_row_lock_waits = lock_sys.get_wait_cumulative();
@@ -1586,7 +1586,7 @@ static bool srv_purge_should_exit()
     return true;
 
   /* Slow shutdown was requested. */
-  const size_t history_size= trx_sys.history_size();
+  const uint32_t history_size= trx_sys.history_size();
   if (history_size)
   {
     static time_t progress_time;
@@ -1596,9 +1596,9 @@ static bool srv_purge_should_exit()
       progress_time= now;
 #if defined HAVE_SYSTEMD && !defined EMBEDDED_LIBRARY
       service_manager_extend_timeout(INNODB_EXTEND_TIMEOUT_INTERVAL,
-				     "InnoDB: to purge %zu transactions",
+				     "InnoDB: to purge %u transactions",
 				     history_size);
-      sql_print_information("InnoDB: to purge %zu transactions", history_size);
+      ib::info() << "to purge " << history_size << " transactions";
 #endif
     }
     return false;
@@ -1895,7 +1895,7 @@ static void srv_shutdown_purge_tasks()
   std::unique_lock<std::mutex> lk(purge_thd_mutex);
   while (!purge_thds.empty())
   {
-    destroy_background_thd(purge_thds.front());
+    innobase_destroy_background_thd(purge_thds.front());
     purge_thds.pop_front();
   }
   n_purge_thds= 0;
@@ -1944,7 +1944,8 @@ void srv_purge_shutdown()
 		while(!srv_purge_should_exit()) {
 			ut_a(!purge_sys.paused());
 			srv_wake_purge_thread_if_not_active();
-			purge_coordinator_task.wait();
+			std::this_thread::sleep_for(
+				std::chrono::milliseconds(1));
 		}
 		purge_sys.coordinator_shutdown();
 		srv_shutdown_purge_tasks();
