@@ -251,22 +251,20 @@ err_exit:
 
 /** Rename the redo log file after resizing.
 @return whether an error occurred */
-bool log_t::rename_resized() noexcept
+bool log_t::resize_rename() noexcept
 {
-  ut_ad(!srv_log_file_created);
-  ut_d(srv_log_file_created= true);
-
   std::string old_name{get_log_file_path("ib_logfile101")};
   std::string new_name{get_log_file_path()};
 
   if (IF_WIN(MoveFileEx(old_name.c_str(), new_name.c_str(),
-                         MOVEFILE_REPLACE_EXISTING),
+                        MOVEFILE_REPLACE_EXISTING),
              !rename(old_name.c_str(), new_name.c_str())))
     return false;
 
-  sql_print_error("InnoDB: Failed to rename log from %.*s to %.*s",
+  sql_print_error("InnoDB: Failed to rename log from %.*s to %.*s (error %d)",
                   int(old_name.size()), old_name.data(),
-                  int(new_name.size()), new_name.data());
+                  int(new_name.size()), new_name.data(),
+                  IF_WIN(int(GetLastError()), errno));
   return true;
 }
 
@@ -289,12 +287,12 @@ static dberr_t srv_undo_tablespace_create(const char* name)
 
 	if (!ret) {
 		if (os_file_get_last_error(false) != OS_FILE_ALREADY_EXISTS
-#ifdef _AIX
+#ifdef UNIV_AIX
 			/* AIX 5.1 after security patch ML7 may have
 			errno set to 0 here, which causes our function
 			to return 100; work around that AIX problem */
 		    && os_file_get_last_error(false) != 100
-#endif
+#endif /* UNIV_AIX */
 		) {
 			ib::error() << "Can't create UNDO tablespace "
 				<< name;
@@ -649,12 +647,10 @@ dberr_t srv_undo_tablespaces_init(bool create_new_db)
     mtr_t mtr;
     for (uint32_t i= 0; i < srv_undo_tablespaces; ++i)
     {
-      mtr.start();
-      dberr_t err= fsp_header_init(fil_space_get(srv_undo_space_id_start + i),
-                                   SRV_UNDO_TABLESPACE_SIZE_IN_PAGES, &mtr);
-      mtr.commit();
-      if (err)
-        return err;
+       mtr.start();
+       fsp_header_init(fil_space_get(srv_undo_space_id_start + i),
+		       SRV_UNDO_TABLESPACE_SIZE_IN_PAGES, &mtr);
+       mtr.commit();
     }
   }
 
@@ -698,13 +694,10 @@ srv_open_tmp_tablespace(bool create_new_db)
 		mtr_t mtr;
 		mtr.start();
 		mtr.set_log_mode(MTR_LOG_NO_REDO);
-		err = fsp_header_init(fil_system.temp_space,
-				      srv_tmp_space.get_sum_of_sizes(),
-				      &mtr);
+		fsp_header_init(fil_system.temp_space,
+				srv_tmp_space.get_sum_of_sizes(),
+				&mtr);
 		mtr.commit();
-		if (err == DB_SUCCESS) {
-			err = trx_temp_rseg_create(&mtr);
-		}
 	} else {
 		/* This file was just opened in the code above! */
 		ib::error() << "The innodb_temporary"
@@ -732,21 +725,6 @@ static void srv_shutdown_threads()
 	}
 }
 
-
-/** Shut down background threads that can generate undo log. */
-static void srv_shutdown_bg_undo_sources()
-{
-  srv_shutdown_state= SRV_SHUTDOWN_INITIATED;
-
-  if (srv_undo_sources)
-  {
-    ut_ad(!srv_read_only_mode);
-    fts_optimize_shutdown();
-    dict_stats_shutdown();
-    srv_undo_sources= false;
-  }
-}
-
 #ifdef UNIV_DEBUG
 # define srv_init_abort(_db_err)	\
 	srv_init_abort_low(create_new_db, __FILE__, __LINE__, _db_err)
@@ -1157,18 +1135,17 @@ dberr_t srv_start(bool create_new_db)
 		ut_ad(fil_system.sys_space->id == 0);
 		compile_time_assert(TRX_SYS_SPACE == 0);
 		compile_time_assert(IBUF_SPACE_ID == 0);
-		ut_a(fsp_header_init(fil_system.sys_space,
-				     uint32_t(sum_of_new_sizes), &mtr)
-		     == DB_SUCCESS);
+		fsp_header_init(fil_system.sys_space,
+				uint32_t(sum_of_new_sizes), &mtr);
 
 		ulint ibuf_root = btr_create(
 			DICT_CLUSTERED | DICT_IBUF, fil_system.sys_space,
-			DICT_IBUF_ID_MIN, nullptr, &mtr, &err);
+			DICT_IBUF_ID_MIN, nullptr, &mtr);
 
 		mtr_commit(&mtr);
 
 		if (ibuf_root == FIL_NULL) {
-			return srv_init_abort(err);
+			return(srv_init_abort(DB_ERROR));
 		}
 
 		ut_ad(ibuf_root == IBUF_TREE_ROOT_PAGE_NO);
@@ -1177,7 +1154,8 @@ dberr_t srv_start(bool create_new_db)
 		the first rollback segment before the double write buffer.
 		All the remaining rollback segments will be created later,
 		after the double write buffer has been created. */
-		err = trx_sys_create_sys_pages(&mtr);
+		trx_sys_create_sys_pages();
+		err = trx_lists_init_at_db_start();
 
 		if (err != DB_SUCCESS) {
 			return(srv_init_abort(err));
@@ -1191,7 +1169,10 @@ dberr_t srv_start(bool create_new_db)
 
 		buf_flush_sync();
 
-		if (log_sys.rename_resized()) {
+		ut_ad(!srv_log_file_created);
+		ut_d(srv_log_file_created= true);
+
+		if (log_sys.resize_rename()) {
 			return(srv_init_abort(DB_ERROR));
 		}
 	} else {
@@ -1282,10 +1263,6 @@ dberr_t srv_start(bool create_new_db)
 				buf_block_t* block = buf_page_get(
 					page_id_t(0, 0), 0,
 					RW_SX_LATCH, &mtr);
-				/* The first page of the system tablespace
-				should already have been successfully
-				accessed earlier during startup. */
-				ut_a(block);
 				ulint size = mach_read_from_4(
 					FSP_HEADER_OFFSET + FSP_SIZE
 					+ block->page.frame);
@@ -1396,7 +1373,7 @@ dberr_t srv_start(bool create_new_db)
 
 			err = create_log_file(false, lsn);
 
-			if (err == DB_SUCCESS && log_sys.rename_resized()) {
+			if (err == DB_SUCCESS && log_sys.resize_rename()) {
 				err = DB_ERROR;
 			}
 
@@ -1447,11 +1424,6 @@ dberr_t srv_start(bool create_new_db)
 				page_id_t(IBUF_SPACE_ID,
 					  FSP_IBUF_HEADER_PAGE_NO),
 				0, RW_X_LATCH, &mtr);
-			if (UNIV_UNLIKELY(!block)) {
-			corrupted_old_page:
-				mtr.commit();
-				return srv_init_abort(DB_CORRUPTION);
-			}
 			fil_block_check_type(*block, FIL_PAGE_TYPE_SYS, &mtr);
 			/* Already MySQL 3.23.53 initialized
 			FSP_IBUF_TREE_ROOT_PAGE_NO to
@@ -1459,25 +1431,16 @@ dberr_t srv_start(bool create_new_db)
 			block = buf_page_get(
 				page_id_t(TRX_SYS_SPACE, TRX_SYS_PAGE_NO),
 				0, RW_X_LATCH, &mtr);
-			if (UNIV_UNLIKELY(!block)) {
-				goto corrupted_old_page;
-			}
 			fil_block_check_type(*block, FIL_PAGE_TYPE_TRX_SYS,
 					     &mtr);
 			block = buf_page_get(
 				page_id_t(TRX_SYS_SPACE,
 					  FSP_FIRST_RSEG_PAGE_NO),
 				0, RW_X_LATCH, &mtr);
-			if (UNIV_UNLIKELY(!block)) {
-				goto corrupted_old_page;
-			}
 			fil_block_check_type(*block, FIL_PAGE_TYPE_SYS, &mtr);
 			block = buf_page_get(
 				page_id_t(TRX_SYS_SPACE, FSP_DICT_HDR_PAGE_NO),
 				0, RW_X_LATCH, &mtr);
-			if (UNIV_UNLIKELY(!block)) {
-				goto corrupted_old_page;
-			}
 			fil_block_check_type(*block, FIL_PAGE_TYPE_SYS, &mtr);
 			mtr.commit();
 		}
@@ -1577,6 +1540,8 @@ skip_monitors:
 			return(srv_init_abort(err));
 		}
 
+		trx_temp_rseg_create();
+
 		if (srv_force_recovery < SRV_FORCE_NO_BACKGROUND) {
 			srv_start_periodic_timer(srv_master_timer, srv_master_callback, 1000);
 		}
@@ -1645,6 +1610,19 @@ skip_monitors:
 	return(DB_SUCCESS);
 }
 
+/** Shut down background threads that can generate undo log. */
+void srv_shutdown_bg_undo_sources()
+{
+	srv_shutdown_state = SRV_SHUTDOWN_INITIATED;
+
+	if (srv_undo_sources) {
+		ut_ad(!srv_read_only_mode);
+		fts_optimize_shutdown();
+		dict_stats_shutdown();
+		srv_undo_sources = false;
+	}
+}
+
 /**
   Shutdown purge to make sure that there is no possibility that we call any
   plugin code (e.g., audit) inside virtual column computation.
