@@ -14,7 +14,7 @@
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA */
 
-#define SPIDER_DB_WRAPPER_STR "mysql"
+#define SPIDER_DB_WRAPPER_STR "mariadb"
 #define SPIDER_DB_WRAPPER_LEN (sizeof(SPIDER_DB_WRAPPER_STR) - 1)
 #define SPIDER_DB_PK_NAME_STR "PRIMARY"
 #define SPIDER_DB_PK_NAME_LEN (sizeof(SPIDER_DB_PK_NAME_STR) - 1)
@@ -34,6 +34,7 @@
 #define SPIDER_DB_INSERT_IGNORE        (1 << 1)
 #define SPIDER_DB_INSERT_LOW_PRIORITY  (1 << 2)
 #define SPIDER_DB_INSERT_HIGH_PRIORITY (1 << 3)
+#define SPIDER_DB_INSERT_DELAYED       (1 << 4)
 
 #define SPIDER_SQL_OPEN_PAREN_STR "("
 #define SPIDER_SQL_OPEN_PAREN_LEN (sizeof(SPIDER_SQL_OPEN_PAREN_STR) - 1)
@@ -79,6 +80,8 @@
 #define SPIDER_SQL_HIGH_PRIORITY_LEN (sizeof(SPIDER_SQL_HIGH_PRIORITY_STR) - 1)
 #define SPIDER_SQL_LOW_PRIORITY_STR "low_priority "
 #define SPIDER_SQL_LOW_PRIORITY_LEN (sizeof(SPIDER_SQL_LOW_PRIORITY_STR) - 1)
+#define SPIDER_SQL_SQL_DELAYED_STR "delayed "
+#define SPIDER_SQL_SQL_DELAYED_LEN (sizeof(SPIDER_SQL_SQL_DELAYED_STR) - 1)
 #define SPIDER_SQL_SQL_IGNORE_STR "ignore "
 #define SPIDER_SQL_SQL_IGNORE_LEN (sizeof(SPIDER_SQL_SQL_IGNORE_STR) - 1)
 #define SPIDER_SQL_FROM_STR " from "
@@ -496,7 +499,6 @@ int spider_db_append_charset_name_before_string(
   CHARSET_INFO *cs
 );
 
-#ifdef HANDLER_HAS_DIRECT_AGGREGATE
 int spider_db_refetch_for_item_sum_funcs(
   ha_spider *spider
 );
@@ -511,7 +513,6 @@ int spider_db_fetch_for_item_sum_func(
   Item_sum *item_sum,
   ha_spider *spider
 );
-#endif
 
 int spider_db_append_match_fetch(
   ha_spider *spider,
@@ -636,14 +637,6 @@ void spider_db_discard_multiple_result(
   SPIDER_CONN *conn
 );
 
-#ifdef HA_CAN_BULK_ACCESS
-int spider_db_bulk_store_result(
-  ha_spider *spider,
-  SPIDER_CONN *conn,
-  int link_idx,
-  bool discard_result
-);
-#endif
 
 int spider_db_fetch(
   uchar *buf,
@@ -766,11 +759,6 @@ int spider_db_bulk_insert(
   bool bulk_end
 );
 
-#ifdef HA_CAN_BULK_ACCESS
-int spider_db_bulk_bulk_insert(
-  ha_spider *spider
-);
-#endif
 
 int spider_db_update_auto_increment(
   ha_spider *spider,
@@ -799,32 +787,13 @@ int spider_db_update(
   const uchar *old_data
 );
 
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS_WITH_HS
-int spider_db_direct_update(
-  ha_spider *spider,
-  TABLE *table,
-  KEY_MULTI_RANGE *ranges,
-  uint range_count,
-  ha_rows *update_rows,
-  ha_rows *found_rows
-);
-#else
 int spider_db_direct_update(
   ha_spider *spider,
   TABLE *table,
   ha_rows *update_rows,
   ha_rows *found_rows
 );
-#endif
-#endif
 
-#ifdef HA_CAN_BULK_ACCESS
-int spider_db_bulk_direct_update(
-  ha_spider *spider,
-  ha_rows *update_rows
-);
-#endif
 
 int spider_db_bulk_delete(
   ha_spider *spider,
@@ -838,23 +807,11 @@ int spider_db_delete(
   const uchar *buf
 );
 
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS_WITH_HS
-int spider_db_direct_delete(
-  ha_spider *spider,
-  TABLE *table,
-  KEY_MULTI_RANGE *ranges,
-  uint range_count,
-  ha_rows *delete_rows
-);
-#else
 int spider_db_direct_delete(
   ha_spider *spider,
   TABLE *table,
   ha_rows *delete_rows
 );
-#endif
-#endif
 
 int spider_db_delete_all_rows(
   ha_spider *spider
@@ -944,7 +901,6 @@ int spider_db_open_item_func(
   spider_fields *fields
 );
 
-#ifdef HANDLER_HAS_DIRECT_AGGREGATE
 int spider_db_open_item_sum_func(
   Item_sum *item_sum,
   ha_spider *spider,
@@ -955,7 +911,6 @@ int spider_db_open_item_sum_func(
   bool use_fields,
   spider_fields *fields
 );
-#endif
 
 int spider_db_open_item_ident(
   Item_ident *item_ident,
@@ -1056,7 +1011,6 @@ int spider_db_append_condition(
   bool test_flg
 );
 
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
 int spider_db_append_update_columns(
   ha_spider *spider,
   spider_string *str,
@@ -1066,14 +1020,11 @@ int spider_db_append_update_columns(
   bool use_fields,
   spider_fields *fields
 );
-#endif
 
-#ifdef HANDLER_HAS_DIRECT_AGGREGATE
 bool spider_db_check_select_colum_in_group(
   st_select_lex *select_lex,
   Field *field
 );
-#endif
 
 uint spider_db_check_ft_idx(
   Item_func *item_func,
@@ -1202,13 +1153,6 @@ int spider_db_open_handler(
   int link_idx
 );
 
-#ifdef HA_CAN_BULK_ACCESS
-int spider_db_bulk_open_handler(
-  ha_spider *spider,
-  SPIDER_CONN *conn,
-  int link_idx
-);
-#endif
 
 int spider_db_close_handler(
   ha_spider *spider,
@@ -1217,12 +1161,6 @@ int spider_db_close_handler(
   uint tgt_conn_kind
 );
 
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-void spider_db_hs_request_buf_reset(
-  SPIDER_CONN *conn
-);
-#endif
-
 bool spider_db_conn_is_network_error(
   int error_num
 );
