@@ -18,14 +18,9 @@
 #include <my_global.h>
 #include "mysql_version.h"
 #include "spd_environ.h"
-#if MYSQL_VERSION_ID < 50500
-#include "mysql_priv.h"
-#include <mysql/plugin.h>
-#else
 #include "sql_priv.h"
 #include "probes_mysql.h"
 #include "sql_class.h"
-#endif
 #include "sql_common.h"
 #include <mysql.h>
 #include <errmsg.h>
@@ -45,7 +40,6 @@ spider_db_result::spider_db_result(
   DBUG_VOID_RETURN;
 }
 
-#ifdef HA_HAS_CHECKSUM_EXTENDED
 int spider_db_result::fetch_table_checksum(
   ha_spider *spider
 ) {
@@ -53,7 +47,6 @@ int spider_db_result::fetch_table_checksum(
   DBUG_PRINT("info",("spider this=%p", this));
   DBUG_RETURN(0);
 }
-#endif
 
 uint spider_db_result::limit_mode()
 {
@@ -141,21 +134,42 @@ bool spider_db_util::tables_on_different_db_are_joinable()
 {
   DBUG_ENTER("spider_db_util::tables_on_different_db_are_joinable");
   DBUG_PRINT("info",("spider this=%p", this));
-  DBUG_RETURN(TRUE);
+  DBUG_RETURN(FALSE);
 }
 
 bool spider_db_util::socket_has_default_value()
 {
   DBUG_ENTER("spider_db_util::socket_has_default_value");
   DBUG_PRINT("info",("spider this=%p", this));
-  DBUG_RETURN(TRUE);
+  DBUG_RETURN(FALSE);
 }
 
 bool spider_db_util::database_has_default_value()
 {
   DBUG_ENTER("spider_db_util::database_has_default_value");
   DBUG_PRINT("info",("spider this=%p", this));
-  DBUG_RETURN(TRUE);
+  DBUG_RETURN(FALSE);
+}
+
+bool spider_db_util::default_file_has_default_value()
+{
+  DBUG_ENTER("spider_db_util::default_file_has_default_value");
+  DBUG_PRINT("info",("spider this=%p", this));
+  DBUG_RETURN(FALSE);
+}
+
+bool spider_db_util::host_has_default_value()
+{
+  DBUG_ENTER("spider_db_util::host_has_default_value");
+  DBUG_PRINT("info",("spider this=%p", this));
+  DBUG_RETURN(FALSE);
+}
+
+bool spider_db_util::port_has_default_value()
+{
+  DBUG_ENTER("spider_db_util::port_has_default_value");
+  DBUG_PRINT("info",("spider this=%p", this));
+  DBUG_RETURN(FALSE);
 }
 
 bool spider_db_util::append_charset_name_before_string()
@@ -172,7 +186,6 @@ uint spider_db_util::limit_mode()
   DBUG_RETURN(0);
 }
 
-#ifdef HA_HAS_CHECKSUM_EXTENDED
 bool spider_db_share::checksum_support()
 {
   DBUG_ENTER("spider_db_share::checksum_support");
@@ -187,9 +200,7 @@ int spider_db_handler::checksum_table(
   DBUG_PRINT("info",("spider this=%p", this));
   DBUG_RETURN(0);
 }
-#endif
 
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
 bool spider_db_handler::check_direct_update(
   st_select_lex *select_lex,
   longlong select_limit,
@@ -223,4 +234,3 @@ bool spider_db_handler::check_direct_delete(
   }
   DBUG_RETURN(FALSE);
 }
-#endif
