@@ -1,5 +1,5 @@
 /* Copyright (C) 2008-2020 Kentoku Shiba
-   Copyright (C) 2019, 2022, MariaDB Corporation.
+   Copyright (C) 2019-2020 MariaDB corp
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -28,56 +28,20 @@
 #define SPIDER_DB_WRAPPER_MYSQL "mysql"
 #define SPIDER_DB_WRAPPER_MARIADB "mariadb"
 
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100204
 #define PLUGIN_VAR_CAN_MEMALLOC
-/*
-#define ITEM_FUNC_CASE_PARAMS_ARE_PUBLIC
-#define HASH_UPDATE_WITH_HASH_VALUE
-*/
-#else
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
-#define HANDLER_HAS_DIRECT_UPDATE_ROWS_WITH_HS
-#endif
-#endif
 
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100002
-#define SPIDER_HAS_DISCOVER_TABLE_STRUCTURE
 #define SPIDER_HAS_APPEND_FOR_SINGLE_QUOTE
 #define SPIDER_HAS_SHOW_SIMPLE_FUNC
 #define SPIDER_HAS_JT_HASH_INDEX_MERGE
 #define SPIDER_HAS_EXPR_CACHE_ITEM
-#else
-#define SPIDER_NEED_CHECK_CONDITION_AT_CHECKING_DIRECT_ORDER_LIMIT
-#endif
-
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100007
 #define SPIDER_ITEM_HAS_CMP_TYPE
-#endif
-
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100004
 #define SPIDER_HAS_TIME_STATUS
 #define SPIDER_HAS_DECIMAL_OPERATION_RESULTS_VALUE_TYPE
-#endif
-
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100014
 #define SPIDER_ITEM_STRING_WITHOUT_SET_STR_WITH_COPY
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100100
 #define SPIDER_ITEM_STRING_WITHOUT_SET_STR_WITH_COPY_AND_THDPTR
-#endif
-#endif
-
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100108
-#define SPIDER_HAS_GROUP_BY_HANDLER
-#endif
-
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100200
 #define SPIDER_ORDER_HAS_ENUM_ORDER
-#endif
-
-#if defined(MARIADB_BASE_VERSION)
 #define SPIDER_ITEM_GEOFUNC_NAME_HAS_MBR
 #define SPIDER_HANDLER_AUTO_REPAIR_HAS_ERROR
-#endif
 
 class spider_db_conn;
 typedef spider_db_conn SPIDER_DB_CONN;
@@ -101,8 +65,6 @@ typedef st_spider_result SPIDER_RESULT;
 
 #define SPIDER_SQL_DOT_STR "."
 #define SPIDER_SQL_DOT_LEN (sizeof(SPIDER_SQL_DOT_STR) - 1)
-#define SPIDER_SQL_PERCENT_STR "%"
-#define SPIDER_SQL_PERCENT_LEN (sizeof(SPIDER_SQL_PERCENT_STR) - 1)
 #define SPIDER_SQL_HYPHEN_STR "-"
 #define SPIDER_SQL_HYPHEN_LEN (sizeof(SPIDER_SQL_HYPHEN_STR) - 1)
 
@@ -154,8 +116,6 @@ typedef st_spider_result SPIDER_RESULT;
 #define SPIDER_SQL_IN_LEN (sizeof(SPIDER_SQL_IN_STR) - 1)
 #define SPIDER_SQL_NOT_IN_STR "not in("
 #define SPIDER_SQL_NOT_IN_LEN (sizeof(SPIDER_SQL_NOT_IN_STR) - 1)
-#define SPIDER_SQL_LIKE_STR " like "
-#define SPIDER_SQL_LIKE_LEN (sizeof(SPIDER_SQL_LIKE_STR) - 1)
 #define SPIDER_SQL_NOT_LIKE_STR "not like"
 #define SPIDER_SQL_NOT_LIKE_LEN (sizeof(SPIDER_SQL_NOT_LIKE_STR) - 1)
 #define SPIDER_SQL_AS_CHAR_STR " as char"
@@ -760,10 +720,13 @@ public:
   virtual int append_escaped_name_quote(
     spider_string *str
   ) = 0;
-  virtual int append_column_value(ha_spider *spider, spider_string *str,
-                                  Field *field, const uchar *new_ptr,
-                                  bool is_like,
-                                  CHARSET_INFO *access_charset)= 0;
+  virtual int append_column_value(
+    ha_spider *spider,
+    spider_string *str,
+    Field *field,
+    const uchar *new_ptr,
+    CHARSET_INFO *access_charset
+  ) = 0;
   virtual int append_trx_isolation(
     spider_string *str,
     int trx_isolation
@@ -827,7 +790,6 @@ public:
     bool use_fields,
     spider_fields *fields
   ) = 0;
-#ifdef HANDLER_HAS_DIRECT_AGGREGATE
   virtual int open_item_sum_func(
     Item_sum *item_sum,
     ha_spider *spider,
@@ -837,12 +799,10 @@ public:
     bool use_fields,
     spider_fields *fields
   ) = 0;
-#endif
   virtual int append_escaped_util(
     spider_string *to,
     String *from
   ) = 0;
-#ifdef SPIDER_HAS_GROUP_BY_HANDLER
   virtual int append_from_and_tables(
     ha_spider *spider,
     spider_fields *fields,
@@ -861,7 +821,6 @@ public:
   virtual int append_having(
     spider_string *str
   ) = 0;
-#endif
   virtual bool tables_on_different_db_are_joinable();
   virtual bool socket_has_default_value();
   virtual bool database_has_default_value();
@@ -944,11 +903,9 @@ public:
     int mode,
     ha_rows &records
   ) = 0;
-#ifdef HA_HAS_CHECKSUM_EXTENDED
   virtual int fetch_table_checksum(
     ha_spider *spider
   );
-#endif
   virtual int fetch_table_cardinality(
     int mode,
     TABLE *table,
@@ -965,7 +922,6 @@ public:
     longlong pos
   ) = 0;
   virtual int get_errno() = 0;
-#ifdef SPIDER_HAS_DISCOVER_TABLE_STRUCTURE
   virtual int fetch_columns_for_discover_table_structure(
     spider_string *str,
     CHARSET_INFO *access_charset
@@ -979,7 +935,6 @@ public:
     SPIDER_SHARE *spider_share,
     CHARSET_INFO *access_charset
   ) = 0;
-#endif
   virtual uint limit_mode();
 };
 
@@ -1186,16 +1141,12 @@ public:
     uint alias_length
   ) = 0;
   virtual bool need_change_db_table_name() = 0;
-#ifdef SPIDER_HAS_DISCOVER_TABLE_STRUCTURE
   virtual int discover_table_structure(
     SPIDER_TRX *trx,
     SPIDER_SHARE *spider_share,
     spider_string *str
   ) = 0;
-#endif
-#ifdef HA_HAS_CHECKSUM_EXTENDED
   virtual bool checksum_support();
-#endif
 };
 
 class spider_db_handler
@@ -1210,9 +1161,7 @@ public:
   ha_spider *spider;
   spider_db_share *db_share;
   int first_link_idx;
-#ifdef SPIDER_HAS_GROUP_BY_HANDLER
   SPIDER_LINK_IDX_CHAIN *link_idx_chain;
-#endif
   bool strict_group_by;
   bool no_where_cond;
   spider_db_handler(ha_spider *spider, spider_db_share *db_share) :
@@ -1264,7 +1213,6 @@ public:
   virtual int append_update_part() = 0;
   virtual int append_delete_part() = 0;
   virtual int append_update_set_part() = 0;
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
   virtual int append_direct_update_set_part() = 0;
   virtual int append_dup_update_pushdown_part(
     const char *alias,
@@ -1275,7 +1223,6 @@ public:
     uint alias_length
   ) = 0;
   virtual int check_update_columns_part() = 0;
-#endif
   virtual int append_select_part(
     ulong sql_type
   ) = 0;
@@ -1357,26 +1304,22 @@ public:
     const char *alias,
     uint alias_length
   ) = 0;
-#ifdef HANDLER_HAS_DIRECT_AGGREGATE
   virtual int append_sum_select_part(
     ulong sql_type,
     const char *alias,
     uint alias_length
   ) = 0;
-#endif
   virtual void set_order_pos(
     ulong sql_type
   ) = 0;
   virtual void set_order_to_pos(
     ulong sql_type
   ) = 0;
-#ifdef HANDLER_HAS_DIRECT_AGGREGATE
   virtual int append_group_by_part(
     const char *alias,
     uint alias_length,
     ulong sql_type
   ) = 0;
-#endif
   virtual int append_key_order_for_merge_with_alias_part(
     const char *alias,
     uint alias_length,
@@ -1504,13 +1447,11 @@ public:
   virtual bool need_lock_before_set_sql_for_exec(
     ulong sql_type
   ) = 0;
-#ifdef SPIDER_HAS_GROUP_BY_HANDLER
   virtual int set_sql_for_exec(
     ulong sql_type,
     int link_idx,
     SPIDER_LINK_IDX_CHAIN *link_idx_chain
   ) = 0;
-#endif
   virtual int set_sql_for_exec(
     ulong sql_type,
     int link_idx
@@ -1544,11 +1485,9 @@ public:
   virtual int show_records(
     int link_idx
   ) = 0;
-#ifdef HA_HAS_CHECKSUM_EXTENDED
   virtual int checksum_table(
     int link_idx
   );
-#endif
   virtual int show_last_insert_id(
     int link_idx,
     ulonglong &last_insert_id
@@ -1626,7 +1565,6 @@ public:
     int link_idx,
     ulong sql_type
   ) = 0;
-#ifdef SPIDER_HAS_GROUP_BY_HANDLER
   virtual int append_from_and_tables_part(
     spider_fields *fields,
     ulong sql_type
@@ -1673,8 +1611,6 @@ public:
     spider_fields *fields,
     ulong sql_type
   ) = 0;
-#endif
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
   virtual bool check_direct_update(
     st_select_lex *select_lex,
     longlong select_limit,
@@ -1685,7 +1621,6 @@ public:
     longlong select_limit,
     longlong offset_limit
   );
-#endif
 };
 
 class spider_db_copy_table
@@ -1804,9 +1739,7 @@ typedef struct st_spider_position
   uint                   pos_mode;
   bool                   use_position;
   bool                   mrr_with_cnt;
-#ifdef HANDLER_HAS_DIRECT_AGGREGATE
   bool                   direct_aggregate;
-#endif
   uint                   sql_kind;
   uchar                  *position_bitmap;
   st_spider_ft_info      *ft_first;
@@ -1825,13 +1758,9 @@ typedef struct st_spider_result
 {
   uint                 dbton_id;
   SPIDER_DB_RESULT     *result;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   volatile
-#endif
     st_spider_result   *prev;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   volatile
-#endif
     st_spider_result   *next;
   SPIDER_POSITION      *first_position; /* for quick mode */
   int                  pos_page_size; /* for quick mode */
@@ -1850,17 +1779,11 @@ typedef struct st_spider_result
 
 typedef struct st_spider_result_list
 {
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   volatile
-#endif
     SPIDER_RESULT        *first;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   volatile
-#endif
     SPIDER_RESULT        *last;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   volatile
-#endif
     SPIDER_RESULT         *current;
   KEY                     *key_info;
   int                     key_order;
@@ -1907,12 +1830,10 @@ typedef struct st_spider_result_list
   /* the limit_offeset, without where condition */
   bool                    direct_limit_offset;
   bool                    direct_distinct;
-#ifdef HANDLER_HAS_DIRECT_AGGREGATE
   bool                    direct_aggregate;
   bool                    snap_mrr_with_cnt;
   bool                    snap_direct_aggregate;
   SPIDER_DB_ROW           *snap_row;
-#endif
   bool                    in_cmp_ref;
   bool                    set_split_read;
   bool                    insert_dup_update_pushdown;
@@ -1924,14 +1845,11 @@ typedef struct st_spider_result_list
   longlong                second_read;
   int                     set_split_read_count;
   int                     *casual_read;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   /* 0:nomal 1:store 2:store end */
   volatile
-#endif
     int                   quick_phase;
   bool                    keyread;
   TABLE                   *table;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   volatile int            bgs_error;
   bool                    bgs_error_with_message;
   char                    bgs_error_msg[MYSQL_ERRMSG_SIZE];
@@ -1942,7 +1860,6 @@ typedef struct st_spider_result_list
   volatile longlong       bgs_second_read;
   volatile longlong       bgs_split_read;
   volatile
-#endif
     SPIDER_RESULT         *bgs_current;
   SPIDER_DB_ROW           *tmp_pos_row_first;
 } SPIDER_RESULT_LIST;
