@@ -1,4 +1,5 @@
-/* Copyright (C) 2008-2018 Kentoku Shiba
+/* Copyright (C) 2008-2019 Kentoku Shiba
+   Copyright (C) 2019 MariaDB corp
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -17,16 +18,11 @@
 #include <my_global.h>
 #include "mysql_version.h"
 #include "spd_environ.h"
-#if MYSQL_VERSION_ID < 50500
-#include "mysql_priv.h"
-#include <mysql/plugin.h>
-#else
 #include "sql_priv.h"
 #include "probes_mysql.h"
 #include "sql_class.h"
 #include "sql_partition.h"
 #include "ha_partition.h"
-#endif
 #include "sql_common.h"
 #include <errmsg.h>
 #include "spd_err.h"
@@ -637,9 +633,9 @@ SPIDER_CONN_HOLDER *spider_fields::create_conn_holder(
   DBUG_PRINT("info",("spider this=%p", this));
   return_conn_holder = (SPIDER_CONN_HOLDER *)
     spider_bulk_malloc(spider_current_trx, 252, MYF(MY_WME | MY_ZEROFILL),
-      &return_conn_holder, sizeof(SPIDER_CONN_HOLDER),
+      &return_conn_holder, (uint) (sizeof(SPIDER_CONN_HOLDER)),
       &table_link_idx_holder,
-        table_count * sizeof(SPIDER_TABLE_LINK_IDX_HOLDER),
+        (uint) (table_count * sizeof(SPIDER_TABLE_LINK_IDX_HOLDER)),
       NullS
     );
   if (!return_conn_holder)
@@ -794,7 +790,7 @@ void spider_fields::choose_a_conn(
   SPIDER_CONN_HOLDER *conn_holder;
   longlong balance_total = 0, balance_val;
   double rand_val;
-  THD *thd = table_holder[0].spider->trx->thd;
+  THD *thd = table_holder[0].spider->wide_handler->trx->thd;
   DBUG_ENTER("spider_fields::choose_a_conn");
   DBUG_PRINT("info",("spider this=%p", this));
   for (current_conn_holder = first_conn_holder; current_conn_holder;
@@ -1146,8 +1142,8 @@ int spider_fields::ping_table_mon_from_table(
     if (tmp_share->monitoring_kind[tmp_link_idx])
     {
       error_num_buf = spider_ping_table_mon_from_table(
-          tmp_spider->trx,
-          tmp_spider->trx->thd,
+          tmp_spider->wide_handler->trx,
+          tmp_spider->wide_handler->trx->thd,
           tmp_share,
           tmp_link_idx,
           (uint32) tmp_share->monitoring_sid[tmp_link_idx],
@@ -1168,7 +1164,6 @@ int spider_fields::ping_table_mon_from_table(
   DBUG_RETURN(error_num);
 }
 
-#ifdef SPIDER_HAS_GROUP_BY_HANDLER
 spider_group_by_handler::spider_group_by_handler(
   THD *thd_arg,
   Query *query_arg,
@@ -1180,7 +1175,7 @@ spider_group_by_handler::spider_group_by_handler(
   fields->set_pos_to_first_table_holder();
   SPIDER_TABLE_HOLDER *table_holder = fields->get_next_table_holder();
   spider = table_holder->spider;
-  trx = spider->trx;
+  trx = spider->wide_handler->trx;
   DBUG_VOID_RETURN;
 }
 
@@ -1246,24 +1241,20 @@ int spider_group_by_handler::init_scan()
   for (link_idx = 0; link_idx < (int) share->link_count; ++link_idx)
     spider->sql_kind[link_idx] = SPIDER_SQL_KIND_SQL;
 
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
   spider->do_direct_update = FALSE;
   spider->direct_update_kinds = 0;
-#endif
   spider_get_select_limit(spider, &select_lex, &select_limit, &offset_limit);
   direct_order_limit = spider_param_direct_order_limit(thd,
     share->direct_order_limit);
   if (
     direct_order_limit &&
-    select_lex->explicit_limit &&
+    select_lex->limit_params.explicit_limit &&
     !(select_lex->options & OPTION_FOUND_ROWS) &&
     select_limit < direct_order_limit /* - offset_limit */
   ) {
     result_list->internal_limit = select_limit /* + offset_limit */;
     result_list->split_read = select_limit /* + offset_limit */;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
     result_list->bgs_split_read = select_limit /* + offset_limit */;
-#endif
 
     result_list->split_read_base = 9223372036854775807LL;
     result_list->semi_split_read = 0;
@@ -1274,10 +1265,8 @@ int spider_group_by_handler::init_scan()
   }
   result_list->semi_split_read_base = 0;
   result_list->set_split_read = TRUE;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   if ((error_num = spider_set_conn_bg_param(spider)))
     DBUG_RETURN(error_num);
-#endif
   DBUG_PRINT("info",("spider result_list.finish_flg = FALSE"));
   result_list->finish_flg = FALSE;
   result_list->record_num = 0;
@@ -1290,7 +1279,7 @@ int spider_group_by_handler::init_scan()
     result_list->internal_limit >= result_list->split_read ?
     result_list->split_read : result_list->internal_limit;
 
-  if (select_lex->explicit_limit)
+  if (select_lex->limit_params.explicit_limit)
   {
     result_list->internal_offset += offset_limit;
   } else {
@@ -1385,7 +1374,6 @@ int spider_group_by_handler::init_scan()
     link_idx = link_idx_holder->link_idx;
     dbton_hdl = spider->dbton_handler[conn->dbton_id];
     spider->link_idx_chain = link_idx_chain;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
     if (result_list->bgs_phase > 0)
     {
       if ((error_num = spider_check_and_init_casual_read(trx->thd, spider,
@@ -1409,7 +1397,6 @@ int spider_group_by_handler::init_scan()
         DBUG_RETURN(error_num);
       }
     } else {
-#endif
       pthread_mutex_assert_not_owner(&conn->mta_conn_mutex);
       if (dbton_hdl->need_lock_before_set_sql_for_exec(
         SPIDER_SQL_TYPE_SELECT_SQL))
@@ -1515,9 +1502,7 @@ int spider_group_by_handler::init_scan()
         SPIDER_CLEAR_FILE_POS(&conn->mta_conn_mutex_file_pos);
         pthread_mutex_unlock(&conn->mta_conn_mutex);
       }
-#ifndef WITHOUT_SPIDER_BG_SEARCH
     }
-#endif
   }
 
   first = TRUE;
@@ -1556,7 +1541,6 @@ int spider_group_by_handler::next_row()
           table->status = STATUS_NOT_FOUND;
         DBUG_RETURN(spider->store_error_num);
       }
-#ifndef WITHOUT_SPIDER_BG_SEARCH
       if (spider->result_list.bgs_phase > 0)
       {
         fields->set_pos_to_first_link_idx_chain();
@@ -1585,7 +1569,6 @@ int spider_group_by_handler::next_row()
           }
         }
       }
-#endif
       spider->use_pre_call = FALSE;
     }
   } else if (offset_limit)
@@ -1645,7 +1628,6 @@ group_by_handler *spider_create_group_by_handler(
       break;
   }
 
-#ifdef WITH_PARTITION_STORAGE_ENGINE
   from = query->from;
   do {
     DBUG_PRINT("info",("spider from=%p", from));
@@ -1654,23 +1636,16 @@ group_by_handler *spider_create_group_by_handler(
     if (from->table->part_info)
     {
       DBUG_PRINT("info",("spider partition handler"));
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
       partition_info *part_info = from->table->part_info;
       uint bits = bitmap_bits_set(&part_info->read_partitions);
       DBUG_PRINT("info",("spider bits=%u", bits));
       if (bits != 1)
       {
         DBUG_PRINT("info",("spider using multiple partitions is not supported by this feature yet"));
-#else
-        DBUG_PRINT("info",("spider partition is not supported by this feature yet"));
-#endif
         DBUG_RETURN(NULL);
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
       }
-#endif
     }
   } while ((from = from->next_local));
-#endif
 
   table_idx = 0;
   from = query->from;
@@ -1683,7 +1658,6 @@ group_by_handler *spider_create_group_by_handler(
     /* all tables are const_table */
     DBUG_RETURN(NULL);
   }
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
   if (from->table->part_info)
   {
     partition_info *part_info = from->table->part_info;
@@ -1692,11 +1666,8 @@ group_by_handler *spider_create_group_by_handler(
     handler **handlers = partition->get_child_handlers();
     spider = (ha_spider *) handlers[part];
   } else {
-#endif
     spider = (ha_spider *) from->table->file;
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
   }
-#endif
   share = spider->share;
   spider->idx_for_direct_join = table_idx;
   ++table_idx;
@@ -1715,7 +1686,6 @@ group_by_handler *spider_create_group_by_handler(
   {
     if (from->table->const_table)
       continue;
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
     if (from->table->part_info)
     {
       partition_info *part_info = from->table->part_info;
@@ -1724,11 +1694,8 @@ group_by_handler *spider_create_group_by_handler(
       handler **handlers = partition->get_child_handlers();
       spider = (ha_spider *) handlers[part];
     } else {
-#endif
       spider = (ha_spider *) from->table->file;
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
     }
-#endif
     share = spider->share;
     spider->idx_for_direct_join = table_idx;
     ++table_idx;
@@ -1754,7 +1721,6 @@ group_by_handler *spider_create_group_by_handler(
   do {
     if (from->table->const_table)
       continue;
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
     if (from->table->part_info)
     {
       partition_info *part_info = from->table->part_info;
@@ -1763,11 +1729,8 @@ group_by_handler *spider_create_group_by_handler(
       handler **handlers = partition->get_child_handlers();
       spider = (ha_spider *) handlers[part];
     } else {
-#endif
       spider = (ha_spider *) from->table->file;
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
     }
-#endif
     share = spider->share;
     if (spider_param_skip_default_condition(thd,
       share->skip_default_condition))
@@ -1908,7 +1871,6 @@ group_by_handler *spider_create_group_by_handler(
   {
     from = from->next_local;
   }
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
   if (from->table->part_info)
   {
     partition_info *part_info = from->table->part_info;
@@ -1917,11 +1879,8 @@ group_by_handler *spider_create_group_by_handler(
     handler **handlers = partition->get_child_handlers();
     spider = (ha_spider *) handlers[part];
   } else {
-#endif
     spider = (ha_spider *) from->table->file;
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
   }
-#endif
   share = spider->share;
   lock_mode = spider_conn_lock_mode(spider);
   if (lock_mode)
@@ -1938,6 +1897,12 @@ group_by_handler *spider_create_group_by_handler(
     delete fields;
     DBUG_RETURN(NULL);
   }
+  if (spider->dml_init())
+  {
+    DBUG_PRINT("info",("spider can not init for dml"));
+    delete fields;
+    DBUG_RETURN(NULL);
+  }
   for (
     roop_count = spider_conn_link_idx_next(share->link_statuses,
       spider->conn_link_idx, -1, share->link_count,
@@ -1997,7 +1962,6 @@ group_by_handler *spider_create_group_by_handler(
       continue;
     fields->clear_conn_holder_from_conn();
 
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
     if (from->table->part_info)
     {
       partition_info *part_info = from->table->part_info;
@@ -2006,11 +1970,8 @@ group_by_handler *spider_create_group_by_handler(
       handler **handlers = partition->get_child_handlers();
       spider = (ha_spider *) handlers[part];
     } else {
-#endif
       spider = (ha_spider *) from->table->file;
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
     }
-#endif
     share = spider->share;
     if (!fields->add_table(spider))
     {
@@ -2020,6 +1981,12 @@ group_by_handler *spider_create_group_by_handler(
     }
     DBUG_PRINT("info",("spider s->db=%s", from->table->s->db.str));
     DBUG_PRINT("info",("spider s->table_name=%s", from->table->s->table_name.str));
+    if (spider->dml_init())
+    {
+      DBUG_PRINT("info",("spider can not init for dml"));
+      delete fields;
+      DBUG_RETURN(NULL);
+    }
     for (
       roop_count = spider_conn_link_idx_next(share->link_statuses,
         spider->conn_link_idx, -1, share->link_count,
@@ -2128,4 +2095,3 @@ group_by_handler *spider_create_group_by_handler(
   query->order_by = NULL;
   DBUG_RETURN(group_by_handler);
 }
-#endif
