@@ -17,15 +17,10 @@
 #include <my_global.h>
 #include "mysql_version.h"
 #include "spd_environ.h"
-#if MYSQL_VERSION_ID < 50500
-#include "mysql_priv.h"
-#include <mysql/plugin.h>
-#else
 #include "sql_priv.h"
 #include "probes_mysql.h"
 #include "sql_class.h"
 #include "sql_partition.h"
-#endif
 #include <my_getopt.h>
 #include "spd_err.h"
 #include "spd_db_include.h"
@@ -36,15 +31,12 @@
 
 extern struct st_mysql_plugin spider_i_s_alloc_mem;
 extern struct st_mysql_plugin spider_i_s_wrapper_protocols;
-#ifdef MARIADB_BASE_VERSION
 extern struct st_maria_plugin spider_i_s_alloc_mem_maria;
 extern struct st_maria_plugin spider_i_s_wrapper_protocols_maria;
-#endif
 
 extern volatile ulonglong spider_mon_table_cache_version;
 extern volatile ulonglong spider_mon_table_cache_version_req;
 
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
 static int spider_direct_update(THD *thd, SHOW_VAR *var, char *buff)
 {
   int error_num = 0;
@@ -66,7 +58,6 @@ static int spider_direct_delete(THD *thd, SHOW_VAR *var, char *buff)
     var->value = (char *) &trx->direct_delete_count;
   DBUG_RETURN(error_num);
 }
-#endif
 
 static int spider_direct_order_limit(THD *thd, SHOW_VAR *var, char *buff)
 {
@@ -107,7 +98,6 @@ struct st_mysql_show_var spider_status_variables[] =
     (char *) &spider_mon_table_cache_version, SHOW_LONGLONG},
   {"Spider_mon_table_cache_version_req",
     (char *) &spider_mon_table_cache_version_req, SHOW_LONGLONG},
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
 #ifdef SPIDER_HAS_SHOW_SIMPLE_FUNC
   {"Spider_direct_update", (char *) &spider_direct_update, SHOW_SIMPLE_FUNC},
   {"Spider_direct_delete", (char *) &spider_direct_delete, SHOW_SIMPLE_FUNC},
@@ -115,7 +105,6 @@ struct st_mysql_show_var spider_status_variables[] =
   {"Spider_direct_update", (char *) &spider_direct_update, SHOW_FUNC},
   {"Spider_direct_delete", (char *) &spider_direct_delete, SHOW_FUNC},
 #endif
-#endif
 #ifdef SPIDER_HAS_SHOW_SIMPLE_FUNC
   {"Spider_direct_order_limit",
     (char *) &spider_direct_order_limit, SHOW_SIMPLE_FUNC},
@@ -135,13 +124,8 @@ struct st_mysql_show_var spider_status_variables[] =
 };
 
 typedef DECLARE_MYSQL_THDVAR_SIMPLE(thdvar_int_t, int);
-#if MYSQL_VERSION_ID < 50500
-extern bool throw_bounds_warning(THD *thd, bool fixed, bool unsignd,
-  const char *name, long long val);
-#else
 extern bool throw_bounds_warning(THD *thd, const char *name, bool fixed,
   bool is_unsignd, longlong v);
-#endif
 
 static my_bool spider_support_xa;
 static MYSQL_SYSVAR_BOOL(
@@ -154,6 +138,20 @@ static MYSQL_SYSVAR_BOOL(
   TRUE
 );
 
+static void spider_use_table_value_deprecated(THD *thd, st_mysql_sys_var *,
+                                              void *var_ptr, const void *save)
+{
+  int val= *static_cast<const int *>(save);
+  *static_cast<int *>(var_ptr)= val;
+  if (val == -1)
+  {
+    push_warning_printf(thd, Sql_condition::WARN_LEVEL_WARN,
+                        HA_ERR_UNSUPPORTED,
+                        "The option value -1 (use table value) is deprecated "
+                        "and will be removed in a future release");
+  }
+}
+
 my_bool spider_param_support_xa()
 {
   DBUG_ENTER("spider_param_support_xa");
@@ -235,8 +233,8 @@ static MYSQL_SYSVAR_INT(
   PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_READONLY,
   "Use table charset for remote access",
   NULL,
-  NULL,
-  -1,
+  spider_use_table_value_deprecated,
+  1,
   -1,
   1,
   0
@@ -412,7 +410,7 @@ uint spider_param_force_commit(
  */
 static MYSQL_THDVAR_UINT(
   xa_register_mode, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED, /* opt */
   "Mode of XA transaction register into system table", /* comment */
   NULL, /* check */
   NULL, /* update */
@@ -435,11 +433,11 @@ uint spider_param_xa_register_mode(
  */
 static MYSQL_THDVAR_LONGLONG(
   internal_offset, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED, /* opt */
   "Internal offset", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  0, /* def */
   -1, /* min */
   9223372036854775807LL, /* max */
   0 /* blk */
@@ -460,11 +458,11 @@ longlong spider_param_internal_offset(
  */
 static MYSQL_THDVAR_LONGLONG(
   internal_limit, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED, /* opt */
   "Internal limit", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  9223372036854775807LL, /* def */
   -1, /* min */
   9223372036854775807LL, /* max */
   0 /* blk */
@@ -488,8 +486,8 @@ static MYSQL_THDVAR_LONGLONG(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Number of rows at a select", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  9223372036854775807LL, /* def */
   -1, /* min */
   9223372036854775807LL, /* max */
   0 /* blk */
@@ -514,8 +512,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Use offset and limit parameter in SQL for split_read parameter.", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  2, /* def */
   -1, /* min */
   2147483647, /* max */
   0 /* blk */
@@ -539,8 +537,8 @@ static MYSQL_THDVAR_LONGLONG(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "The limit value for semi_split_read", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  9223372036854775807LL, /* def */
   -1, /* min */
   9223372036854775807LL, /* max */
   0 /* blk */
@@ -565,8 +563,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Initial sql string alloc size", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  1024, /* def */
   -1, /* min */
   2147483647, /* max */
   0 /* blk */
@@ -591,8 +589,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Reset sql string alloc after execute", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  1, /* def */
   -1, /* min */
   1, /* max */
   0 /* blk */
@@ -617,8 +615,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Sprit read mode for multi range", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  100, /* def */
   -1, /* min */
   2147483647, /* max */
   0 /* blk */
@@ -642,8 +640,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Max columns for order by", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  32767, /* def */
   -1, /* min */
   32767, /* max */
   0 /* blk */
@@ -714,14 +712,9 @@ static int spider_param_semi_table_lock_check(
     (long) ((MYSQL_SYSVAR_NAME(thdvar_int_t) *) var)->blk_sz;
   options.arg_type = REQUIRED_ARG;
   *((int *) save) = (int) getopt_ll_limit_value(tmp, &options, &fixed);
-#if MYSQL_VERSION_ID < 50500
-  DBUG_RETURN(throw_bounds_warning(thd, fixed, FALSE,
-    ((MYSQL_SYSVAR_NAME(thdvar_int_t) *) var)->name, (long long) tmp));
-#else
   DBUG_RETURN(throw_bounds_warning(thd,
     ((MYSQL_SYSVAR_NAME(thdvar_int_t) *) var)->name, fixed, FALSE,
     (longlong) tmp));
-#endif
 }
 
 /*
@@ -778,14 +771,9 @@ static int spider_param_semi_table_lock_connection_check(
     (long) ((MYSQL_SYSVAR_NAME(thdvar_int_t) *) var)->blk_sz;
   options.arg_type = REQUIRED_ARG;
   *((int *) save) = (int) getopt_ll_limit_value(tmp, &options, &fixed);
-#if MYSQL_VERSION_ID < 50500
-  DBUG_RETURN(throw_bounds_warning(thd, fixed, FALSE,
-    ((MYSQL_SYSVAR_NAME(thdvar_int_t) *) var)->name, (long long) tmp));
-#else
   DBUG_RETURN(throw_bounds_warning(thd,
     ((MYSQL_SYSVAR_NAME(thdvar_int_t) *) var)->name, fixed, FALSE,
     (longlong) tmp));
-#endif
 }
 
 /*
@@ -798,8 +786,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Use different connection if semi_table_lock is enabled", /* comment */
   &spider_param_semi_table_lock_connection_check, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  1, /* def */
   -1, /* min */
   1, /* max */
   0 /* blk */
@@ -847,8 +835,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Lock for select with update", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  1, /* def */
   -1, /* min */
   2, /* max */
   0 /* blk */
@@ -936,8 +924,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Bulk insert size", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  16000, /* def */
   -1, /* min */
   2147483647, /* max */
   0 /* blk */
@@ -965,8 +953,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "The mode of bulk updating and deleting", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  0, /* def */
   -1, /* min */
   2, /* max */
   0 /* blk */
@@ -990,8 +978,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Bulk update size", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  16000, /* def */
   -1, /* min */
   2147483647, /* max */
   0 /* blk */
@@ -1015,8 +1003,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Buffer size", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  16000, /* def */
   -1, /* min */
   2147483647, /* max */
   0 /* blk */
@@ -1041,8 +1029,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Execute optimize to remote server", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  0, /* def */
   -1, /* min */
   1, /* max */
   0 /* blk */
@@ -1067,8 +1055,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Execute optimize to remote server with local", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  0, /* def */
   -1, /* min */
   1, /* max */
   0 /* blk */
@@ -1216,8 +1204,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Wait timeout of connecting to remote server", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  6, /* def */
   -1, /* min */
   2147483647, /* max */
   0 /* blk */
@@ -1243,8 +1231,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Wait timeout of receiving data from remote server", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  600, /* def */
   -1, /* min */
   2147483647, /* max */
   0 /* blk */
@@ -1270,8 +1258,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Wait timeout of sending data to remote server", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  600, /* def */
   -1, /* min */
   2147483647, /* max */
   0 /* blk */
@@ -1301,8 +1289,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "The retrieval result from a remote server is acquired by acquisition one by one", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  3, /* def */
   -1, /* min */
   3, /* max */
   0 /* blk */
@@ -1326,8 +1314,8 @@ static MYSQL_THDVAR_LONGLONG(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Number of records in a page when acquisition one by one", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  1024, /* def */
   -1, /* min */
   9223372036854775807LL, /* max */
   0 /* blk */
@@ -1351,8 +1339,8 @@ static MYSQL_THDVAR_LONGLONG(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "The limitation of memory size in a page when acquisition one by one", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  10485760, /* def */
   -1, /* min */
   9223372036854775807LL, /* max */
   0 /* blk */
@@ -1377,8 +1365,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Use low memory mode when SQL(SELECT) internally issued to a remote server is executed and get a result list", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  1, /* def */
   -1, /* min */
   1, /* max */
   0 /* blk */
@@ -1404,8 +1392,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "The mode of using columns at select clause", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  1, /* def */
   -1, /* min */
   1, /* max */
   0 /* blk */
@@ -1420,7 +1408,6 @@ int spider_param_select_column_mode(
     select_column_mode : THDVAR(thd, select_column_mode));
 }
 
-#ifndef WITHOUT_SPIDER_BG_SEARCH
 /*
  -1 :use table parameter
   0 :background search is disabled
@@ -1433,8 +1420,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Mode of background search", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  0, /* def */
   -1, /* min */
   3, /* max */
   0 /* blk */
@@ -1459,8 +1446,8 @@ static MYSQL_THDVAR_LONGLONG(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Number of first read records when background search is used", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  2, /* def */
   -1, /* min */
   9223372036854775807LL, /* max */
   0 /* blk */
@@ -1485,8 +1472,8 @@ static MYSQL_THDVAR_LONGLONG(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Number of second read records when background search is used", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  100, /* def */
   -1, /* min */
   9223372036854775807LL, /* max */
   0 /* blk */
@@ -1500,7 +1487,6 @@ longlong spider_param_bgs_second_read(
   DBUG_RETURN(THDVAR(thd, bgs_second_read) < 0 ?
     bgs_second_read : THDVAR(thd, bgs_second_read));
 }
-#endif
 
 /*
  -1 :use table parameter
@@ -1512,8 +1498,8 @@ static MYSQL_THDVAR_LONGLONG(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Number of first read records", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  0, /* def */
   -1, /* min */
   9223372036854775807LL, /* max */
   0 /* blk */
@@ -1538,8 +1524,8 @@ static MYSQL_THDVAR_LONGLONG(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Number of second read records", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  0, /* def */
   -1, /* min */
   9223372036854775807LL, /* max */
   0 /* blk */
@@ -1564,8 +1550,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Interval of cardinality confirmation.(second)", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  51, /* def */
   -1, /* min */
   2147483647, /* max */
   0 /* blk */
@@ -1589,11 +1575,11 @@ double spider_param_crd_interval(
  */
 static MYSQL_THDVAR_INT(
   crd_mode, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED, /* opt */
   "Mode of cardinality confirmation.", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  1, /* def */
   -1, /* min */
   3, /* max */
   0 /* blk */
@@ -1608,7 +1594,6 @@ int spider_param_crd_mode(
     crd_mode : THDVAR(thd, crd_mode));
 }
 
-#ifdef WITH_PARTITION_STORAGE_ENGINE
 /*
  -1 :use table parameter
   0 :No synchronization.
@@ -1621,8 +1606,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Cardinality synchronization in partitioned table.", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  0, /* def */
   -1, /* min */
   2, /* max */
   0 /* blk */
@@ -1636,7 +1621,6 @@ int spider_param_crd_sync(
   DBUG_RETURN(THDVAR(thd, crd_sync) == -1 ?
     crd_sync : THDVAR(thd, crd_sync));
 }
-#endif
 
 /*
  -1 :use table parameter
@@ -1646,11 +1630,11 @@ int spider_param_crd_sync(
  */
 static MYSQL_THDVAR_INT(
   crd_type, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED, /* opt */
   "Type of cardinality calculation.", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  2, /* def */
   -1, /* min */
   2, /* max */
   0 /* blk */
@@ -1671,11 +1655,11 @@ int spider_param_crd_type(
  */
 static MYSQL_THDVAR_INT(
   crd_weight, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED, /* opt */
   "Weight coefficient to calculate effectiveness of index from cardinality of column.", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  2, /* def */
   -1, /* min */
   2147483647, /* max */
   0 /* blk */
@@ -1690,7 +1674,6 @@ double spider_param_crd_weight(
     crd_weight : THDVAR(thd, crd_weight));
 }
 
-#ifndef WITHOUT_SPIDER_BG_SEARCH
 /*
  -1 :use table parameter
   0 :Background confirmation is disabled
@@ -1702,8 +1685,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Mode of cardinality confirmation at background.", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  2, /* def */
   -1, /* min */
   2, /* max */
   0 /* blk */
@@ -1717,7 +1700,6 @@ int spider_param_crd_bg_mode(
   DBUG_RETURN(THDVAR(thd, crd_bg_mode) == -1 ?
     crd_bg_mode : THDVAR(thd, crd_bg_mode));
 }
-#endif
 
 /*
  -1 :use table parameter
@@ -1729,8 +1711,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Interval of table state confirmation.(second)", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  10, /* def */
   -1, /* min */
   2147483647, /* max */
   0 /* blk */
@@ -1753,11 +1735,11 @@ double spider_param_sts_interval(
  */
 static MYSQL_THDVAR_INT(
   sts_mode, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED, /* opt */
   "Mode of table state confirmation.", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  1, /* def */
   -1, /* min */
   2, /* max */
   0 /* blk */
@@ -1772,7 +1754,6 @@ int spider_param_sts_mode(
     sts_mode : THDVAR(thd, sts_mode));
 }
 
-#ifdef WITH_PARTITION_STORAGE_ENGINE
 /*
  -1 :use table parameter
   0 :No synchronization.
@@ -1785,8 +1766,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Table state synchronization in partitioned table.", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  0, /* def */
   -1, /* min */
   2, /* max */
   0 /* blk */
@@ -1800,9 +1781,7 @@ int spider_param_sts_sync(
   DBUG_RETURN(THDVAR(thd, sts_sync) == -1 ?
     sts_sync : THDVAR(thd, sts_sync));
 }
-#endif
 
-#ifndef WITHOUT_SPIDER_BG_SEARCH
 /*
  -1 :use table parameter
   0 :Background confirmation is disabled
@@ -1814,8 +1793,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Mode of table state confirmation at background.", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  2, /* def */
   -1, /* min */
   2, /* max */
   0 /* blk */
@@ -1829,7 +1808,6 @@ int spider_param_sts_bg_mode(
   DBUG_RETURN(THDVAR(thd, sts_bg_mode) == -1 ?
     sts_bg_mode : THDVAR(thd, sts_bg_mode));
 }
-#endif
 
 /*
   0 :always ping
@@ -1865,8 +1843,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Mode of auto increment.", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  0, /* def */
   -1, /* min */
   3, /* max */
   0 /* blk */
@@ -1958,8 +1936,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Execute \"REPLACE\" and \"INSERT IGNORE\" on remote server and avoid duplicate check on local server", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  0, /* def */
   -1, /* min */
   1, /* max */
   0 /* blk */
@@ -2025,7 +2003,7 @@ uint spider_param_udf_table_mon_mutex_count()
  */
 static MYSQL_THDVAR_LONGLONG(
   udf_ds_bulk_insert_rows, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED, /* opt */
   "Number of rows for bulk inserting", /* comment */
   NULL, /* check */
   NULL, /* update */
@@ -2052,7 +2030,7 @@ longlong spider_param_udf_ds_bulk_insert_rows(
  */
 static MYSQL_THDVAR_INT(
   udf_ds_table_loop_mode, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED, /* opt */
   "Table loop mode if the number of tables in table list are less than the number of result sets", /* comment */
   NULL, /* check */
   NULL, /* update */
@@ -2074,7 +2052,6 @@ int spider_param_udf_ds_table_loop_mode(
 static char *spider_remote_access_charset;
 /*
  */
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100000
 static MYSQL_SYSVAR_STR(
   remote_access_charset,
   spider_remote_access_charset,
@@ -2085,30 +2062,6 @@ static MYSQL_SYSVAR_STR(
   NULL,
   NULL
 );
-#else
-#ifdef PLUGIN_VAR_CAN_MEMALLOC
-static MYSQL_SYSVAR_STR(
-  remote_access_charset,
-  spider_remote_access_charset,
-  PLUGIN_VAR_MEMALLOC |
-  PLUGIN_VAR_RQCMDARG,
-  "Set remote access charset at connecting for improvement performance of connection if you know",
-  NULL,
-  NULL,
-  NULL
-);
-#else
-static MYSQL_SYSVAR_STR(
-  remote_access_charset,
-  spider_remote_access_charset,
-  PLUGIN_VAR_RQCMDARG,
-  "Set remote access charset at connecting for improvement performance of connection if you know",
-  NULL,
-  NULL,
-  NULL
-);
-#endif
-#endif
 
 char *spider_param_remote_access_charset()
 {
@@ -2144,7 +2097,6 @@ int spider_param_remote_autocommit()
 static char *spider_remote_time_zone;
 /*
  */
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100000
 static MYSQL_SYSVAR_STR(
   remote_time_zone,
   spider_remote_time_zone,
@@ -2155,30 +2107,6 @@ static MYSQL_SYSVAR_STR(
   NULL,
   NULL
 );
-#else
-#ifdef PLUGIN_VAR_CAN_MEMALLOC
-static MYSQL_SYSVAR_STR(
-  remote_time_zone,
-  spider_remote_time_zone,
-  PLUGIN_VAR_MEMALLOC |
-  PLUGIN_VAR_RQCMDARG,
-  "Set remote time_zone at connecting for improvement performance of connection if you know",
-  NULL,
-  NULL,
-  NULL
-);
-#else
-static MYSQL_SYSVAR_STR(
-  remote_time_zone,
-  spider_remote_time_zone,
-  PLUGIN_VAR_RQCMDARG,
-  "Set remote time_zone at connecting for improvement performance of connection if you know",
-  NULL,
-  NULL,
-  NULL
-);
-#endif
-#endif
 
 char *spider_param_remote_time_zone()
 {
@@ -2241,7 +2169,6 @@ int spider_param_remote_trx_isolation()
 static char *spider_remote_default_database;
 /*
  */
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100000
 static MYSQL_SYSVAR_STR(
   remote_default_database,
   spider_remote_default_database,
@@ -2252,30 +2179,6 @@ static MYSQL_SYSVAR_STR(
   NULL,
   NULL
 );
-#else
-#ifdef PLUGIN_VAR_CAN_MEMALLOC
-static MYSQL_SYSVAR_STR(
-  remote_default_database,
-  spider_remote_default_database,
-  PLUGIN_VAR_MEMALLOC |
-  PLUGIN_VAR_RQCMDARG,
-  "Set remote database at connecting for improvement performance of connection if you know",
-  NULL,
-  NULL,
-  NULL
-);
-#else
-static MYSQL_SYSVAR_STR(
-  remote_default_database,
-  spider_remote_default_database,
-  PLUGIN_VAR_RQCMDARG,
-  "Set remote database at connecting for improvement performance of connection if you know",
-  NULL,
-  NULL,
-  NULL
-);
-#endif
-#endif
 
 char *spider_param_remote_default_database()
 {
@@ -2333,7 +2236,6 @@ int spider_param_connect_retry_count(
 
 /*
  */
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100000
 static MYSQL_THDVAR_STR(
   bka_engine, /* name */
   PLUGIN_VAR_MEMALLOC |
@@ -2343,28 +2245,6 @@ static MYSQL_THDVAR_STR(
   NULL, /* update */
   NULL /* def */
 );
-#else
-#ifdef PLUGIN_VAR_CAN_MEMALLOC
-static MYSQL_THDVAR_STR(
-  bka_engine, /* name */
-  PLUGIN_VAR_MEMALLOC |
-  PLUGIN_VAR_RQCMDARG,
-  "Temporary table's engine for BKA", /* comment */
-  NULL, /* check */
-  NULL, /* update */
-  NULL /* def */
-);
-#else
-static MYSQL_THDVAR_STR(
-  bka_engine, /* name */
-  PLUGIN_VAR_RQCMDARG,
-  "Temporary table's engine for BKA", /* comment */
-  NULL, /* check */
-  NULL, /* update */
-  NULL /* def */
-);
-#endif
-#endif
 
 char *spider_param_bka_engine(
   THD *thd,
@@ -2385,8 +2265,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Mode of BKA for Spider", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  1, /* def */
   -1, /* min */
   2, /* max */
   0 /* blk */
@@ -2409,7 +2289,7 @@ static int spider_udf_ct_bulk_insert_interval;
 static MYSQL_SYSVAR_INT(
   udf_ct_bulk_insert_interval,
   spider_udf_ct_bulk_insert_interval,
-  PLUGIN_VAR_RQCMDARG,
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED,
   "The interval time between bulk insert and next bulk insert at coping",
   NULL,
   NULL,
@@ -2435,7 +2315,7 @@ static longlong spider_udf_ct_bulk_insert_rows;
 static MYSQL_SYSVAR_LONGLONG(
   udf_ct_bulk_insert_rows,
   spider_udf_ct_bulk_insert_rows,
-  PLUGIN_VAR_RQCMDARG,
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED,
   "The number of rows inserted with bulk insert of one time at coping",
   NULL,
   NULL,
@@ -2460,11 +2340,11 @@ longlong spider_param_udf_ct_bulk_insert_rows(
  */
 static MYSQL_THDVAR_INT(
   use_handler, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED, /* opt */
   "Use handler for reading", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  0, /* def */
   -1, /* min */
   3, /* max */
   0 /* blk */
@@ -2489,8 +2369,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Read error mode if error", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  0, /* def */
   -1, /* min */
   1, /* max */
   0 /* blk */
@@ -2515,8 +2395,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Write error mode if error", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  0, /* def */
   -1, /* min */
   1, /* max */
   0 /* blk */
@@ -2541,8 +2421,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Skip generating internal default condition", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  0, /* def */
   -1, /* min */
   1, /* max */
   0 /* blk */
@@ -2569,8 +2449,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Skip parallel search by specific conditions", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  0, /* def */
   -1, /* min */
   3, /* max */
   0 /* blk */
@@ -2595,8 +2475,8 @@ static MYSQL_THDVAR_LONGLONG(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Send 'ORDER BY' and 'LIMIT' to remote server directly", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  9223372036854775807LL, /* def */
   -1, /* min */
   9223372036854775807LL, /* max */
   0 /* blk */
@@ -2621,8 +2501,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Read only", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  0, /* def */
   -1, /* min */
   1, /* max */
   0 /* blk */
@@ -2637,37 +2517,7 @@ int spider_param_read_only_mode(
     read_only_mode : THDVAR(thd, read_only_mode));
 }
 
-#ifdef HA_CAN_BULK_ACCESS
-static int spider_bulk_access_free;
-/*
- -1 :use table parameter
-  0 :in reset
-  1 :in close
- */
-static MYSQL_SYSVAR_INT(
-  bulk_access_free,
-  spider_bulk_access_free,
-  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_READONLY,
-  "Free mode of bulk access resources",
-  NULL,
-  NULL,
-  -1,
-  -1,
-  1,
-  0
-);
 
-int spider_param_bulk_access_free(
-  int bulk_access_free
-) {
-  DBUG_ENTER("spider_param_bulk_access_free");
-  DBUG_RETURN(spider_bulk_access_free == -1 ?
-    bulk_access_free : spider_bulk_access_free);
-}
-#endif
-
-#if MYSQL_VERSION_ID < 50500
-#else
 /*
  -1 :use UDF parameter
   0 :can not use
@@ -2675,7 +2525,7 @@ int spider_param_bulk_access_free(
  */
 static MYSQL_THDVAR_INT(
   udf_ds_use_real_table, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED, /* opt */
   "Use real table for temporary table list", /* comment */
   NULL, /* check */
   NULL, /* update */
@@ -2693,7 +2543,6 @@ int spider_param_udf_ds_use_real_table(
   DBUG_RETURN(THDVAR(thd, udf_ds_use_real_table) == -1 ?
     udf_ds_use_real_table : THDVAR(thd, udf_ds_use_real_table));
 }
-#endif
 
 static my_bool spider_general_log;
 static MYSQL_SYSVAR_BOOL(
@@ -2870,8 +2719,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Read casually if it is possible", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  0, /* def */
   -1, /* min */
   63, /* max */
   0 /* blk */
@@ -2913,8 +2762,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "The type of delete_all_rows", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  1, /* def */
   -1, /* min */
   1, /* max */
   0 /* blk */
@@ -2939,8 +2788,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "The type of temporary table name for bka", /* comment */
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  0, /* def */
   -1, /* min */
   1, /* max */
   0 /* blk */
@@ -2988,11 +2837,11 @@ static int spider_store_last_sts;
 static MYSQL_SYSVAR_INT(
   store_last_sts,
   spider_store_last_sts,
-  PLUGIN_VAR_RQCMDARG,
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED,
   "Store last sts result into system table",
   NULL,
-  NULL,
-  -1,
+  spider_use_table_value_deprecated,
+  1,
   -1,
   1,
   0
@@ -3015,11 +2864,11 @@ static int spider_store_last_crd;
 static MYSQL_SYSVAR_INT(
   store_last_crd,
   spider_store_last_crd,
-  PLUGIN_VAR_RQCMDARG,
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED,
   "Store last crd result into system table",
   NULL,
-  NULL,
-  -1,
+  spider_use_table_value_deprecated,
+  1,
   -1,
   1,
   0
@@ -3042,11 +2891,11 @@ static int spider_load_sts_at_startup;
 static MYSQL_SYSVAR_INT(
   load_sts_at_startup,
   spider_load_sts_at_startup,
-  PLUGIN_VAR_RQCMDARG,
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED,
   "Load sts from system table at startup",
   NULL,
-  NULL,
-  -1,
+  spider_use_table_value_deprecated,
+  1,
   -1,
   1,
   0
@@ -3069,11 +2918,11 @@ static int spider_load_crd_at_startup;
 static MYSQL_SYSVAR_INT(
   load_crd_at_startup,
   spider_load_crd_at_startup,
-  PLUGIN_VAR_RQCMDARG,
+  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_DEPRECATED,
   "Load crd from system table at startup",
   NULL,
-  NULL,
-  -1,
+  spider_use_table_value_deprecated,
+  1,
   -1,
   1,
   0
@@ -3087,7 +2936,6 @@ int spider_param_load_crd_at_startup(
     load_crd_at_startup : spider_load_crd_at_startup);
 }
 
-#ifndef WITHOUT_SPIDER_BG_SEARCH
 static uint spider_table_sts_thread_count;
 /*
   1-: thread count
@@ -3133,7 +2981,6 @@ uint spider_param_table_crd_thread_count()
   DBUG_ENTER("spider_param_table_crd_thread_count");
   DBUG_RETURN(spider_table_crd_thread_count);
 }
-#endif
 
 static int spider_slave_trx_isolation;
 /*
@@ -3242,8 +3089,8 @@ static MYSQL_THDVAR_INT(
   PLUGIN_VAR_RQCMDARG, /* opt */
   "Use columns in select clause strictly for group by clause",
   NULL, /* check */
-  NULL, /* update */
-  -1, /* def */
+  spider_use_table_value_deprecated, /* update */
+  1, /* def */
   -1, /* min */
   1, /* max */
   0 /* blk */
@@ -3310,35 +3157,25 @@ static struct st_mysql_sys_var* spider_system_variables[] = {
   MYSQL_SYSVAR(quick_page_byte),
   MYSQL_SYSVAR(low_mem_read),
   MYSQL_SYSVAR(select_column_mode),
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   MYSQL_SYSVAR(bgs_mode),
   MYSQL_SYSVAR(bgs_first_read),
   MYSQL_SYSVAR(bgs_second_read),
-#endif
   MYSQL_SYSVAR(first_read),
   MYSQL_SYSVAR(second_read),
   MYSQL_SYSVAR(crd_interval),
   MYSQL_SYSVAR(crd_mode),
-#ifdef WITH_PARTITION_STORAGE_ENGINE
   MYSQL_SYSVAR(crd_sync),
-#endif
   MYSQL_SYSVAR(store_last_crd),
   MYSQL_SYSVAR(load_crd_at_startup),
   MYSQL_SYSVAR(crd_type),
   MYSQL_SYSVAR(crd_weight),
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   MYSQL_SYSVAR(crd_bg_mode),
-#endif
   MYSQL_SYSVAR(sts_interval),
   MYSQL_SYSVAR(sts_mode),
-#ifdef WITH_PARTITION_STORAGE_ENGINE
   MYSQL_SYSVAR(sts_sync),
-#endif
   MYSQL_SYSVAR(store_last_sts),
   MYSQL_SYSVAR(load_sts_at_startup),
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   MYSQL_SYSVAR(sts_bg_mode),
-#endif
   MYSQL_SYSVAR(ping_interval_at_trx_start),
   MYSQL_SYSVAR(auto_increment_mode),
   MYSQL_SYSVAR(same_server_link),
@@ -3369,13 +3206,7 @@ static struct st_mysql_sys_var* spider_system_variables[] = {
   MYSQL_SYSVAR(skip_parallel_search),
   MYSQL_SYSVAR(direct_order_limit),
   MYSQL_SYSVAR(read_only_mode),
-#ifdef HA_CAN_BULK_ACCESS
-  MYSQL_SYSVAR(bulk_access_free),
-#endif
-#if MYSQL_VERSION_ID < 50500
-#else
   MYSQL_SYSVAR(udf_ds_use_real_table),
-#endif
   MYSQL_SYSVAR(general_log),
   MYSQL_SYSVAR(index_hint_pushdown),
   MYSQL_SYSVAR(max_connections),
@@ -3390,10 +3221,8 @@ static struct st_mysql_sys_var* spider_system_variables[] = {
   MYSQL_SYSVAR(bka_table_name_type),
   MYSQL_SYSVAR(use_cond_other_than_pk_for_update),
   MYSQL_SYSVAR(connect_error_interval),
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   MYSQL_SYSVAR(table_sts_thread_count),
   MYSQL_SYSVAR(table_crd_thread_count),
-#endif
   MYSQL_SYSVAR(slave_trx_isolation),
   MYSQL_SYSVAR(remote_wait_timeout),
   MYSQL_SYSVAR(wait_timeout),
@@ -3416,15 +3245,12 @@ mysql_declare_plugin(spider)
   spider_status_variables,
   spider_system_variables,
   NULL,
-#if MYSQL_VERSION_ID >= 50600
   0,
-#endif
 },
 spider_i_s_alloc_mem,
 spider_i_s_wrapper_protocols
 mysql_declare_plugin_end;
 
-#ifdef MARIADB_BASE_VERSION
 maria_declare_plugin(spider)
 {
   MYSQL_STORAGE_ENGINE_PLUGIN,
@@ -3444,4 +3270,3 @@ maria_declare_plugin(spider)
 spider_i_s_alloc_mem_maria,
 spider_i_s_wrapper_protocols_maria
 maria_declare_plugin_end;
-#endif
